program main

  use vtype_module
  use fi_module
  use io_profile, only : &
    & IOProfileInqDimLen, IOProfileRead, IOProfileReadPtcl, IOProfileOutputASCII

  implicit none

  integer                 :: NLev

  real(dp), allocatable   :: r_Press(:)
  integer                 :: NMol
  integer , allocatable   :: m_MolNum(:)             ! Molecular number
  real(dp), allocatable   :: r_Temp  (:)
  real(dp), allocatable   :: rm_VMR  (:,:)
  real(dp)                :: SurfTemp


  character(128) :: ctlfn = 'file.conf'

  character(4*extstr) :: InNcFn
  character(4*extstr) :: OutASCIIFn


  integer             , parameter   :: NChar = stdstr
  real(8)             , allocatable :: z_Press      (:)
  integer                           :: NPtcl
  integer             , allocatable :: a_PtclNum    (:)
  character(len=NChar), allocatable :: a_PtclName   (:)
  real(8)             , allocatable :: a_PtclDens   (:)
  real(8)             , allocatable :: za_PtclEffRad(:,:)
  real(8)             , allocatable :: za_PtclMMR   (:,:)

  character(len=extstr) :: Comment

  integer :: FileUnit
  integer :: iostat


  namelist /file_nml/ &
    & InNcFn, OutASCIIFn, Comment

  InNcFn      = ''
  OutASCIIFn  = ''
  Comment     = ''

  call fi_open( CtlFN, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = file_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist, file_nml, is something wrong.'
  end if
  write( 6, nml = file_nml )
  !
  close( FileUnit )


  !
  ! Input
  !
  call IOProfileInqDimLen(      &
    & InNcFN, 'Press', &
    & NLev &
    & )
  call IOProfileInqDimLen(      &
    & InNcFN, 'MolNum', &
    & NMol &
    & )
  call IOProfileInqDimLen(      &
    & InNcFN, 'PtclNum', &
    & NPtcl &
    & )
  allocate( r_Press      (NLev)         )
  allocate( m_MolNum     (NMol)         )
  allocate( r_Temp       (NLev)         )
  allocate( rm_VMR       (NLev,NMol)    )
  allocate( a_PtclNum    (NPtcl)        )
  allocate( a_PtclName   (NPtcl)        )
  allocate( a_PtclDens   (NPtcl)        )
  allocate( z_Press      (NLev-1)       )
  allocate( za_PtclEffRad(NLev-1,NPtcl) )
  allocate( za_PtclMMR   (NLev-1,NPtcl) )
  call IOProfileRead(      &
    & InNcFN, 'Press', 'MolNum', 'Temp', 'Ts', 'VMR', &
    & NLev, NMol, &
    & r_Press, m_MolNum, r_Temp, SurfTemp, rm_VMR     &
    & )
  call IOProfileReadPtcl(      &
    & InNcFN, &
    & NLev-1, NPtcl, &
    & z_Press, a_PtclNum, a_PtclName, a_PtclDens, za_PtclEffRad, za_PtclMMR  &
    & )


  write( 6, * ) '********************'
  write( 6, * ) '********************'
  write( 6, * ) '********************'
  write( 6, * ) 'Mass mixing ratio of particles are set to zero.'
  za_PtclMMR = 0.0d0
  write( 6, * ) '********************'
  write( 6, * ) '********************'
  write( 6, * ) '********************'



  !
  ! Output
  !
  call IOProfileOutputASCII( &
    & OutASCIIFn, Comment, &
    & NLev, NMol, &
    & r_Press, m_MolNum, r_Temp, rm_VMR, SurfTemp,     &
    & NPtcl, &
    & z_Press, a_PtclNum, &
    & a_PtclName, a_PtclDens, za_PtclEffRad, za_PtclMMR  &
    & )


end program main

!------------------------------------------------------------------------------
