module ac_process_module

  use vtype_module

  implicit none

  !
  ! NWaveNumSubLoop : number of wavenumber sub loop
  !                 : (inner most loop for optimization)
  !
  integer, parameter :: NWaveNumSubLoop =   10
!!$  integer, parameter :: NWaveNumSubLoop = 4096 ! for pi-computer


  public :: ac_process

  !----------------------------------------------------------------------------

contains

  !----------------------------------------------------------------------------

  subroutine ac_process(                              &
    & InputFileName1, InputFileName2, OutputFileName, &
    & Factor1, Factor2                                &
    & )

    use ac_io_module


    character(*), intent(in) :: InputFileName1
    character(*), intent(in) :: InputFileName2
    character(*), intent(in) :: OutputFileName
    real(dp)    , intent(in) :: Factor1
    real(dp)    , intent(in) :: Factor2


    ! Local variables
    integer               :: InNcID1
    character(len=extstr) :: CVersion1
    real(dp)              :: DelWaveNum1
    integer               :: NWaveNum1
    integer               :: NMolNum1
    integer               :: NLev1
    real(dp)              :: AbsCoefMissingValue1
    integer , allocatable :: m_MolNum1(:)
    real(dp), allocatable :: z_Press1 (:)
    real(dp), allocatable :: zm_VMR1  (:,:)
    real(dp), allocatable :: z_Temp1  (:)
    integer               :: InNcID2
    character(len=extstr) :: CVersion2
    real(dp)              :: DelWaveNum2
    integer               :: NWaveNum2
    integer               :: NMolNum2
    integer               :: NLev2
    real(dp)              :: AbsCoefMissingValue2
    integer , allocatable :: m_MolNum2(:)
    real(dp), allocatable :: z_Press2 (:)
    real(dp), allocatable :: zm_VMR2  (:,:)
    real(dp), allocatable :: z_Temp2  (:)
    character(len=extstr) :: CVersion
    real(dp)              :: DelWaveNum
    integer               :: NWaveNum
    integer               :: NMolNum
    integer               :: NLev
    real(dp)              :: AbsCoefMissingValue
    integer , allocatable :: m_MolNum(:)
    real(dp), allocatable :: z_Press (:)
    real(dp), allocatable :: zm_VMR  (:,:)
    real(dp), allocatable :: z_Temp  (:)
    real(dp), parameter   :: AbsCoefMissingValueDummy = -123456789.0d0

    real(dp), allocatable :: w_WaveNum1(:)
    real(dp), allocatable :: w_WaveNum2(:)
    real(dp), allocatable :: w_WaveNum (:)
    real(dp), allocatable :: w_AbsCoef1(:)
    real(dp), allocatable :: w_AbsCoef2(:)
    real(dp), allocatable :: w_AbsCoef (:)
    integer               :: iWaveNum
    integer               :: iwnsls
    integer               :: iwnsle
    logical               :: FlagOutputWaveNum = .true.


    integer :: i
    integer :: k
    integer :: m


    ! Preparation for input
    write( 6, * ) 'Opening input files : ', trim( InputFileName1 )
    write( 6, * ) '                    : ', trim( InputFileName2 )
    AbsCoefMissingValue1 = AbsCoefMissingValueDummy
    AbsCoefMissingValue2 = AbsCoefMissingValueDummy
    call ac_io_input_init( &
      & InputFileName1, CVersion1, DelWaveNum1, NMolNum1, NLev1, NWaveNum1, &
      & AbsCoefMissingValue1,                                               &
      & InNcID1                                                             &
      & )
    call ac_io_input_init( &
      & InputFileName2, CVersion2, DelWaveNum2, NMolNum2, NLev2, NWaveNum2, &
      & AbsCoefMissingValue2,                                               &
      & InNcID2                                                             &
      & )
    if ( trim( CVersion1 ) /= trim( CVersion2 ) ) then
      write( 6, * ) 'ERROR : Version is not the same.'
      write( 6, * ) '       version : ', trim( CVersion1 ), trim( CVersion2 )
      stop
    end if
    if ( DelWaveNum1 /= DelWaveNum2 ) then
      write( 6, * ) 'ERROR : Wavenumber interval (old info.) is not the same.'
      write( 6, * ) '       comment_dwn_m-1 : ', DelWaveNum1, DelWaveNum2
      stop
    end if
    if ( ( NMolNum1       /= NMolNum2       ) .or. &
      &  ( NLev1          /= NLev2          ) .or. &
      &  ( NWaveNum1      /= NWaveNum2      ) ) then
      write( 6, * ) 'ERROR : Sizes of arrays are not the same.'
      write( 6, * ) '       NMolNum  : ', NMolNum1 , NMolNum2
      write( 6, * ) '       NLev     : ', NLev1    , NLev2
      write( 6, * ) '       NWaveNum : ', NWaveNum1, NWaveNum2
      stop
    end if
    CVersion            = CVersion1
    DelWaveNum          = DelWaveNum1
    AbsCoefMissingValue = AbsCoefMissingValue1
    NMolNum             = NMolNum1
    NLev                = NLev1
    NWaveNum            = NWaveNum1
    allocate( m_MolNum1( NMolNum ) )
    allocate( z_Press1 ( NLev    ) )
    allocate( zm_VMR1  ( NLev   , NMolNum ) )
    allocate( z_Temp1  ( NLev    ) )
    call ac_io_input_axisetc( &
      & NMolNum, NLev, &
      & m_MolNum1, z_Press1, zm_VMR1, z_Temp1, &
      & InNcID1                                &
      & )
    allocate( m_MolNum2( NMolNum ) )
    allocate( z_Press2 ( NLev    ) )
    allocate( zm_VMR2  ( NLev   , NMolNum ) )
    allocate( z_Temp2  ( NLev    ) )
    call ac_io_input_axisetc( &
      & NMolNum, NLev, &
      & m_MolNum2, z_Press2, zm_VMR2, z_Temp2, &
      & InNcID2                                &
      & )
    do m = 1, NMolNum
      if ( m_MolNum1(m) /= m_MolNum2(m) ) then
        write( 6, * ) 'ERROR : Molecular number is not the same.'
        write( 6, * ) '      ', m
        write( 6, * ) '      ', m_MolNum1
        write( 6, * ) '      ', m_MolNum2
        stop
      end if
    end do
    do k = 1, NLev
      if ( z_Press1(k) /= z_Press2(k) ) then
!!$      if ( abs( z_Press1(k) - z_Press2(k) ) / ( z_Press1(k) + 1.0d-10 ) > 1.0d-10 ) then
        write( 6, * ) 'ERROR : Pressure is not the same.'
        write( 6, * ) '      ', k
        write( 6, * ) '      ', z_Press1
        write( 6, * ) '      ', z_Press2
        stop
      end if
      if ( z_Temp1(k) /= z_Temp2(k) ) then
!!$      if ( abs( z_Temp1(k) - z_Temp2(k) ) / ( z_Temp1(k) ) > 1.0d-10 ) then
        write( 6, * ) 'ERROR : Temperature is not the same.'
        write( 6, * ) '      ', k
        write( 6, * ) '      ', z_Temp1
        write( 6, * ) '      ', z_Temp2
        stop
      end if
    end do
    do m = 1, NMolNum
      do k = 1, NLev
        if ( zm_VMR1(k,m) /= zm_VMR2(k,m) ) then
!!$        if ( abs( zm_VMR1(k,m) - zm_VMR2(k,m) ) / ( zm_VMR1(k,m) + 1.0d-30 ) > 1.0d-10 ) then
          write( 6, * ) 'ERROR : Volume mixing ratio is not the same.'
          write( 6, * ) '      ', k, m
          write( 6, * ) '      ', zm_VMR1(:,m)
          write( 6, * ) '      ', zm_VMR2(:,m)
          stop
        end if
      end do
    end do
    allocate( m_MolNum( NMolNum ) )
    allocate( z_Press ( NLev    ) )
    allocate( zm_VMR  ( NLev   , NMolNum ) )
    allocate( z_Temp  ( NLev    ) )
    m_MolNum = m_MolNum1
    z_Press  = z_Press1
    zm_VMR   = zm_VMR1
    z_Temp   = z_Temp1

    ! Preparation for output
    if ( AbsCoefMissingValue == AbsCoefMissingValueDummy ) then
      call ac_io_output_init( &
        & OutputFileName, CVersion, &
        & NMolNum, m_MolNum, NLev, z_Press, zm_VMR, z_Temp, DelWaveNum &
        & )
    else
      call ac_io_output_init( &
        & OutputFileName, CVersion, &
        & NMolNum, m_MolNum, NLev, z_Press, zm_VMR, z_Temp, DelWaveNum, AbsCoefMissingValue &
        & )
    end if

    deallocate( m_MolNum1, m_MolNum2 )
    deallocate( z_Press1 , z_Press2  )
    deallocate( zm_VMR1  , zm_VMR2   )
    deallocate( z_Temp1  , z_Temp2   )

    allocate( w_WaveNum1( NWaveNumSubLoop ) )
    allocate( w_WaveNum2( NWaveNumSubLoop ) )
    allocate( w_WaveNum ( NWaveNumSubLoop ) )
    allocate( w_AbsCoef1( NWaveNumSubLoop ) )
    allocate( w_AbsCoef2( NWaveNumSubLoop ) )
    allocate( w_AbsCoef ( NWaveNumSubLoop ) )

    do m = 1, NMolNum
      write( 6, '(a)' ) '================================================='
      write( 6, '(a,i2,a,i2)' ) 'm = ', m, ', Molecular number: ', m_MolNum(m)

      do k = 1, NLev
        if( mod( k, max(NLev/10,1) ) == 0 ) then
          write( 6, '(2i6,a,i5,a)' ) k, NLev, &
            & "  ---  ", int( real( k ) / NLev * 100 ), "%"
        end if


        do iWaveNum = 1, NWaveNum, NWaveNumSubLoop

          iwnsls = 1
          iwnsle = min( NWaveNum - ( iWaveNum - 1 ), NWaveNumSubLoop )

          call ac_io_input(                                             &
            & m, k, NWaveNumSubLoop, 'AbsCoef', w_WaveNum1, w_AbsCoef1, &
            & iwnsls, iwnsle, iWaveNum,                                 &
            & InNcID1                                                   &
            & )
          call ac_io_input(                                             &
            & m, k, NWaveNumSubLoop, 'AbsCoef', w_WaveNum2, w_AbsCoef2, &
            & iwnsls, iwnsle, iWaveNum,                                 &
            & InNcID2                                                   &
            & )

          do i = 1, iwnsle-iwnsls+1
            if ( w_WaveNum1(i) /= w_WaveNum2(i) ) then
              write( 6, * ) 'ERROR : A wavenumber is not the same.'
              write( 6 ,* ) '      ', w_WaveNum1(i)
              write( 6 ,* ) '      ', w_WaveNum2(i)
              stop
            end if
          end do

          w_WaveNum(1:iwnsle-iwnsls+1) = w_WaveNum1(1:iwnsle-iwnsls+1)
          w_AbsCoef(1:iwnsle-iwnsls+1) = &
            &   Factor1 * w_AbsCoef1(1:iwnsle-iwnsls+1) &
            & + Factor2 * w_AbsCoef2(1:iwnsle-iwnsls+1)

          call ac_io_output( m, k, NWaveNumSubLoop, &
            & 'AbsCoef', w_WaveNum, w_AbsCoef, &
            & iwnsls, iwnsle, iWaveNum, FlagOutputWaveNum )

        end do

      end do

    end do

    call ac_io_output_end
    call ac_io_input_end( InNcID1 )
    call ac_io_input_end( InNcID2 )


    deallocate( m_MolNum )
    deallocate( z_Press  )
    deallocate( zm_VMR   )
    deallocate( z_Temp   )

    deallocate( w_WaveNum1 )
    deallocate( w_WaveNum2 )
    deallocate( w_WaveNum  )
    deallocate( w_AbsCoef1 )
    deallocate( w_AbsCoef2 )
    deallocate( w_AbsCoef  )


  end subroutine ac_process

  !----------------------------------------------------------------------------

end module ac_process_module

