#!/usr/bin/ruby

nband     = 21*2   # number of bands
nip_init  = 8

pro_path             = '../sort_ac'
$conf_path           = 'sort_ac.conf'
$conf_template_path  = 'template_'+$conf_path
script_rmse_path     = 'disp_comp_1Dresults_v1.rb'
script_rmse_lbl_path = 'disp_comp_1Dresults_v1_LBL.rb'
ipstruct_path        = 'out/ip_structure.txt'
loss_path            = 'out/loss.txt'
loss_lbl_path        = 'out/loss_lbl.txt'


# check and make output directory
dir = 'out'
unless ( File.directory?( dir ) ) then
  print 'making '+dir+' directory'+"\n"
  com = 'mkdir '+dir
  system( com )
end


b_nip = []   # number of integration point in bands
# initialization of number of integration point
for i in 0..(nband-1)
  b_nip << nip_init
end

# create output files
file = File.open(ipstruct_path, 'w')
line = '# iloop  iband  nip'
file.puts line
file.close
#
file = File.open(loss_path, 'w')
line = '# iloop  loss  band#'
file.puts line
file.close
#
file = File.open(loss_lbl_path, 'w')
line = '# iloop  loss  band#'
file.puts line
file.close


def makeconf( nband, b_nip )

  # make configuration file, sort_ac.conf
  com = 'cp ' + $conf_template_path + ' ' + $conf_path
  system( com )
  #  band setting
  file = File.open($conf_path, 'a')
  for i in 1..(nband/2)
    line = '&band_exception_nml KeyMolNum=1,2,3 '+
           'BandNum='+i.to_s+','+
           'GaussRegNBins(1:2)='+b_nip[2*(i-1)].to_s+','+b_nip[2*(i-1)+1].to_s+',  /'
    file.puts line
  end
  file.close

end

def calc_RMSEs( script_rmse_path )

  # calculate RMSEs
  out_path = 'error_script_each.log'
  com = 'ruby '+script_rmse_path+' 2'+' 1> '+out_path+' 2> /dev/null'
  system( com )
  # read RMSEs
  iline = 1
  i = 0
  a_rmses = Array.new(8,0.0)
  File.foreach( out_path ){|line|
    if ( iline >= 7 ) then
      line = line.split
      a_rmses[i] = line[2].to_f # planetary radiation
      i += 1
      a_rmses[i] = line[4].to_f # solar radiation
      i += 1
    end
    iline += 1
  }

  return a_rmses
end


# calculate reference
print 'calculate reference values with the k-dist. model'+"\n"

#   make configuration file, sort_ac.conf
makeconf( nband, b_nip )

#   execute sort_ac
com = pro_path + ' >> error_pro_all.log'
system( com )

#   calculate RMSEs with respect to LBL
a_rmses = calc_RMSEs( script_rmse_lbl_path )

#   output RMSEs with respect to the LBL model
file = File.open(loss_lbl_path, 'a')
loss = a_rmses[0]+a_rmses[1]+a_rmses[2]+a_rmses[3]
line = 0.to_s+' '+loss.to_s+' '+0.to_s
for i in 0..(a_rmses.size-1)
  line += ' '+a_rmses[i].to_s
end
file.puts line
file.close

#   move output files to save directory
dir_ref = 'out.ref'
unless ( File.directory?( dir_ref ) ) then
  print 'making '+dir_ref+' directory'+"\n"
  com = 'mkdir '+dir_ref
  system( com )
end
com = 'mv ./out/*.nc '+dir_ref
system( com )


for iloop in 1..b_nip.sum
  print 'Outermost loop: ', iloop, "\n"
  print '  IPs: ', b_nip, "\n"


  # loop for trials with increased integration points in each band
  ba_rmses     = Array.new(nband) { Array.new(8,0.0) }
  ba_rmses_lbl = Array.new(nband) { Array.new(8,0.0) }
  for iband in 1..nband
#    print '  Trial for band: ', iband, "\n"
    if ( iband == 1 ) then
      print '  Trial for band: '
    else
      print ','
    end
    print iband

    # prepare number of IPs
    b_nip_current = []
    for i in 0..(nband-1)
      b_nip_current << b_nip[i]
    end
    if ( b_nip_current[iband-1] > 1 ) then
      b_nip_current[iband-1] -= 1
      flag_calc = true
    else
      flag_calc = false
    end


    if ( flag_calc ) then
      # make configuration file, sort_ac.conf
      makeconf( nband, b_nip_current )

      # execute sort_ac
      com = pro_path + ' >> error_pro_all.log'
      system( com )

      # calculate RMSEs
      a_rmses = calc_RMSEs( script_rmse_path )
      for i in 0..(a_rmses.size-1)
        ba_rmses[iband-1][i] = a_rmses[i]
      end

      # calculate RMSEs with respect to LBL
      a_rmses = calc_RMSEs( script_rmse_lbl_path )
      for i in 0..(a_rmses.size-1)
        ba_rmses_lbl[iband-1][i] = a_rmses[i]
      end

    else

      for i in 0..(2*4-1)
        ba_rmses    [iband-1][i] = 1.0e10
        ba_rmses_lbl[iband-1][i] = 1.0e10
      end

    end

  end
  print "\n"
#  p ba_rmses

  # check minimum RMSEs
  b_rmses = []
  for iband in 0..(nband-1)
    #      PR.upflx          +SR.upflx          +PR.dnflx          +SR.dnflx
    loss = ba_rmses[iband][0]+ba_rmses[iband][1]+ba_rmses[iband][2]+ba_rmses[iband][3]
    b_rmses << loss
  end
  i_min = b_rmses.index(b_rmses.min)
  print '  Decrease IP in the band #'+(i_min+1).to_s+' (RMSE sum=', b_rmses.min, ')'+"\n"
  # decrease integration point in the (i_min+1)th band
  b_nip[i_min] -= 1
  # check condition
  if ( b_nip[i_min] <= 0 ) then
    print 'Number of integration point in the band #'+(i_min+1).to_s+' will be zero.'+"\n"
    exit
  end

  # output to file
  line = ''
  file = File.open(ipstruct_path, 'a')
  for iband in 0..(nband-1)
    line = iloop.to_s+' '+iband.to_s+' '+b_nip[iband].to_s
    file.puts line
  end
  line = ''
  file.puts line
  file.close
  #
  file = File.open(loss_path, 'a')
  line = iloop.to_s+' '+b_rmses.min.to_s+' '+(i_min+1).to_s
  for i in 0..(ba_rmses[0].size-1)
    line += ' '+ba_rmses[i_min][i].to_s
  end
  file.puts line
  file.close


  file = File.open(loss_lbl_path, 'a')
  loss = ba_rmses_lbl[i_min][0]+ba_rmses_lbl[i_min][1]+ba_rmses_lbl[i_min][2]+ba_rmses_lbl[i_min][3]
  line = iloop.to_s+' '+loss.to_s+' '+(i_min+1).to_s
  for i in 0..(ba_rmses_lbl[0].size-1)
    line += ' '+ba_rmses_lbl[i_min][i].to_s
  end
  file.puts line
  file.close

end
