#!/usr/bin/ruby
require 'parallel'

#nproc = 10  # number of processes

nband     = 21*2   # number of bands
nip_init  = 8

# TESTTESTTESTTESTTEST
print 'TEST bands and IPs are set.', "\n"
nband     = 3*2    # number of bands
nip_init  = 2
print '  Number of band : ', nband, "\n"
print '  Number of IP   : ', nband*nip_init, "\n"
# TESTTESTTESTTESTTEST


r_syms = [
  'Earth_ICRCCM_LW_Case25_TRP_CO2-300ppmv',
  'Earth_ICRCCM_LW_Case26_TRP_CO2-600ppmv',
  'Earth_ICRCCM_LW_Case27_MLS_CO2-300ppmv',
  'Earth_ICRCCM_LW_Case28_MLS_CO2-600ppmv',
  'Earth_ICRCCM_LW_Case29_MLW_CO2-300ppmv',
  'Earth_ICRCCM_LW_Case30_MLW_CO2-600ppmv',
  'Earth_ICRCCM_LW_Case31_SAS_CO2-300ppmv',
  'Earth_ICRCCM_LW_Case32_SAS_CO2-600ppmv',
  'Earth_ICRCCM_LW_Case33_SAW_CO2-300ppmv',
  'Earth_ICRCCM_LW_Case34_SAW_CO2-600ppmv',
]

r_INPROFNCFNs       = []
r_INORGOPTPROPNCFNs = []
r_syms.each { |sym|
  r_INPROFNCFNs       << '../../../prog01.0_mkprofile_ascii/out/'+sym+'.nc'
  r_INORGOPTPROPNCFNs << '../../../prog02.1_calc_ac/out/'+sym+'_ac.nc'
}

nRefProfs = r_INPROFNCFNs.size


pro_path             = 'sort_ac'
$conf_path                       = 'sort_ac.conf'
$conf_template_path              = 'template_'+$conf_path
$conf_sort_ac_file_path          = 'sort_ac_file.conf'
$conf_sort_ac_file_template_path = 'template_'+$conf_sort_ac_file_path
script_rmse_path     = 'disp_comp_1Dresults_v1.rb'
out_rmse_path        = 'error_script_rmse_each.log'
script_rmse_lbl_path = 'disp_comp_1Dresults_v1_LBL.rb'
out_rmse_lbl_path    = 'error_script_rmse_lbl_each.log'
ipstruct_path        = 'out/ip_structure.txt'
loss_path            = 'out/loss.txt'
loss_lbl_path        = 'out/loss_lbl.txt'
$files_dir           = 'files'


b_nip = []   # number of integration point in bands
# initialization of number of integration point
for i in 0..(nband-1)
  b_nip << nip_init
end


def prep_dir( dir_work, flag_quiet = true )

  # create working directory
  unless ( flag_quiet ) then
    print 'creating working directory '+dir_work+"\n"
  end
  com = 'mkdir '+dir_work
  system( com )

  # copy files required to perform calculation
  unless ( flag_quiet ) then
    print 'copying files to '+dir_work+"\n"
  end
  com = 'cp '+$files_dir+'/* '+dir_work
  system( com )

  # enter into the working directory
  unless ( flag_quiet ) then
    print 'cd '+dir_work+"\n"
  end
  Dir.chdir(dir_work)

  # check and make output directory
  dir = 'out'
  unless ( File.directory?( dir ) ) then
    unless ( flag_quiet ) then
      print 'making '+dir+' directory'+"\n"
    end
    com = 'mkdir '+dir
    system( com )
  end

end


def makeconf( nband, b_nip, cINPROFNCFN, cINORGOPTPROPNCFN )

  # make configuration file, sort_ac.conf
  com = 'cp ' + $conf_template_path + ' ' + $conf_path
  system( com )
  #  band setting
  file = File.open($conf_path, 'a')
  # for 1 subregion
  for i in 1..nband
    line = '&band_exception_nml KeyMolNum=1,2,3 '+
           'BandNum='+i.to_s+','+
           'GaussRegNBins(1:1)='+b_nip[i-1].to_s+','+b_nip[i-1].to_s+',  /'
    file.puts line
  end
  # for 2 subregions
#  for i in 1..(nband/2)
#    line = '&band_exception_nml KeyMolNum=1,2,3 '+
#           'BandNum='+i.to_s+','+
#           'GaussRegNBins(1:2)='+b_nip[2*(i-1)].to_s+','+b_nip[2*(i-1)+1].to_s+',  /'
#    file.puts line
#  end
  file.close

  # make configuration file, sort_ac_file.conf
  com = 'cat ' + $conf_sort_ac_file_template_path +
        '| sed -e s#INPROFNCFN#'       + cINPROFNCFN       + '#g' +
        '| sed -e s#INORGOPTPROPNCFN#' + cINORGOPTPROPNCFN + '#g' +
        '> ' + $conf_sort_ac_file_path
  system( com )

end

def read_RMSEs( out_path )

  # read RMSEs
  iline = 1
  i = 0
  a_rmses = Array.new(8,0.0)
  File.foreach( out_path ){|line|
    if ( iline >= 7 ) then
      line = line.split
      a_rmses[i] = line[2].to_f # planetary radiation
      i += 1
      a_rmses[i] = line[4].to_f # solar radiation
      i += 1
    end
    iline += 1
  }

  return a_rmses
end


dir = 'workdir-ref'
if ( File.directory?( dir ) ) then
  print 'Deleting '+dir, "\n"
  com = 'rm -rf '+dir
  system( com )
end
dir = 'out'
if ( File.directory?( dir ) ) then
  print 'Deleting '+dir, "\n"
  com = 'rm -rf '+dir
  system( com )
end


# check and make output directory
dir = 'out'
unless ( File.directory?( dir ) ) then
#  print '  making '+dir+' directory'+"\n"
  com = 'mkdir '+dir
  system( com )
end

# create output files
file = File.open(ipstruct_path, 'w')
line = '# iloop  iband  nip'
file.puts line
file.close
#
file = File.open(loss_path, 'w')
line = '# iloop  nip  loss  band#'
file.puts line
file.close
#
file = File.open(loss_lbl_path, 'w')
line = '# iloop  nip  loss  band#'
file.puts line
file.close


print '====================', "\n"

# calculate reference
print 'calculate reference values with the k-dist. model'+"\n"

ra_rmses = []
#for iRefProf in 0..(nRefProfs-1)
Parallel.each(0..(nRefProfs-1)) do |iRefProf|
  com = 'sleep ' + iRefProf.to_s
  system( com )
  print '  reference profile #', iRefProf, "\n"

  #   prepare working directory
  dir_work = 'workdir-ref-'+iRefProf.to_s
  prep_dir( dir_work )

  #   make configuration file, sort_ac.conf and sort_ac_file.conf
  cINPROFNCFN       = r_INPROFNCFNs[iRefProf]
  cINORGOPTPROPNCFN = r_INORGOPTPROPNCFNs[iRefProf]
  print '    profile   : ', cINPROFNCFN, "\n"
  print '    opt. prop.: ', cINORGOPTPROPNCFN, "\n"
  makeconf( nband, b_nip, cINPROFNCFN, cINORGOPTPROPNCFN )

  #   execute sort_ac
  com = './'+pro_path + ' >> error_pro_all.log'
  system( com )

  # calculate RMSEs with respect to LBL
  com = 'ruby '+script_rmse_lbl_path+' 2'+' 1> '+out_rmse_lbl_path+' 2> /dev/null'
  system( com )

  #   go back to the parent directory
#  print '  exiting '+dir_work+"\n"
  Dir.chdir('..')

end

print '  Checking RMSEs', "\n"
ra_rmses_lbl = Array.new(nRefProfs) { Array.new(8,0.0) }
for iRefProf in 0..(nRefProfs-1)
  dir_work = 'workdir-ref-'+iRefProf.to_s
  Dir.chdir(dir_work)

  # calculate RMSEs with respect to LBL
  a_rmses = read_RMSEs( out_rmse_lbl_path )
  for i in 0..(a_rmses.size-1)
    ra_rmses_lbl[iRefProf][i] = a_rmses[i]
  end

  #   go back to the parent directory
#    print 'exiting '+dir_work+"\n"
  Dir.chdir('..')
end

#   output RMSEs with respect to the LBL model
file = File.open(loss_lbl_path, 'a')
loss = 0.0
for iRefProf in 0..(nRefProfs-1)
  a_rmses = ra_rmses_lbl[iRefProf]
  loss += a_rmses[0]+a_rmses[1]+a_rmses[2]+a_rmses[3]
end
line = 0.to_s+' '+b_nip.sum.to_s+' '+loss.to_s+' '+0.to_s
for iRefProf in 0..(nRefProfs-1)
  a_rmses = ra_rmses_lbl[iRefProf]
  loss = a_rmses[0]+a_rmses[1]+a_rmses[2]+a_rmses[3]
  line += ' '+loss.to_s
end
file.puts line
file.close

# save results for reference profiles
for iRefProf in 0..(nRefProfs-1)
  dir_work = 'workdir-ref-'+iRefProf.to_s
#  com = 'mv ' + dir_work + ' ' + dir_work+'-'+iRefProf.to_s
  com = 'mv ' + dir_work+'/out' + ' ' + 'out.ref-'+iRefProf.to_s
  system( com )

  com = 'rm -rf ' + dir_work
  system( com )
end


for iloop in 1..b_nip.sum
  print '====================', "\n"
  print 'Outermost loop: ', iloop, "\n"
  print '  NIP: ', b_nip.sum, "\n"
  print '  IPs: ', b_nip, "\n"


  a_dir_work = []
  for iband in 0..(nband-1)
    for iRefProf in 0..(nRefProfs-1)
      a_dir_work << 'workdir-'+iband.to_s.rjust(4,"0")+'-'+iRefProf.to_s
    end
  end


  # loop for trials with increased integration points in each band
#  for iband in 0..(nband-1)
#  Parallel.each(0..(nband-1), in_proceses: nproc) do |iband|
#  Parallel.each(0..(nband-1)) do |iband|
  Parallel.each(0..(nband*nRefProfs-1)) do |isubloop|
    iband    = isubloop / nRefProfs
    iRefProf = isubloop.modulo(nRefProfs)

    if ( iRefProf == 0 ) then
      if ( iband == 0 ) then
        print '  Trial for band: '
      else
        print ','
      end
      print iband
    end

    # prepare number of IPs
    b_nip_current = []
    for i in 0..(nband-1)
      b_nip_current << b_nip[i]
    end
    if ( b_nip_current[iband] > 1 ) then
      b_nip_current[iband] -= 1
      flag_calc = true
    else
      flag_calc = false
    end


    # prepare working directory
    dir_work = a_dir_work[isubloop]
    prep_dir( dir_work )

    if ( flag_calc ) then
      # make configuration file, sort_ac.conf and sort_ac_file.conf
      cINPROFNCFN       = r_INPROFNCFNs[iRefProf]
      cINORGOPTPROPNCFN = r_INORGOPTPROPNCFNs[iRefProf]
      makeconf( nband, b_nip_current, cINPROFNCFN, cINORGOPTPROPNCFN )

      # execute sort_ac
      com = './'+pro_path + ' >> error_pro_all.log'
      system( com )

      # calculate RMSEs
      com = 'cp -Rp ' + '../out.ref-'+iRefProf.to_s + ' out.ref'
      system( com )
      com = 'ruby '+script_rmse_path+' 2'+' 1> '+out_rmse_path+' 2> /dev/null'
      system( com )
      com = 'rm -rf out.ref'
      system( com )

      # calculate RMSEs with respect to LBL
      com = 'ruby '+script_rmse_lbl_path+' 2'+' 1> '+out_rmse_lbl_path+' 2> /dev/null'
      system( com )

    end

    # output flag for calculation
    file = File.open('flag.txt', "w")
    file.puts( flag_calc.to_s )
    file.close

    #   go back to the parent directory
#    print 'exiting '+dir_work+"\n"
    Dir.chdir('..')

  end
  print "\n"
#  p ba_rmses


  print '  Checking RMSEs', "\n"
  bra_rmses     = Array.new(nband) { Array.new(nRefProfs) { Array.new(8,0.0) } }
  bra_rmses_lbl = Array.new(nband) { Array.new(nRefProfs) { Array.new(8,0.0) } }
#  for iband in 0..(nband-1)
  for isubloop in 0..(nband*nRefProfs-1)
    iband    = isubloop / nRefProfs
    iRefProf = isubloop.modulo(nRefProfs)
    dir_work = a_dir_work[isubloop]
    Dir.chdir(dir_work)

    flag_calc = true
    file = File.open('flag.txt', "r")
    flag_calc = file.read
    file.close
    flag_calc.chomp!
    if ( flag_calc == 'true' ) then
      flag_calc = true
    else
      flag_calc = false
    end

    if ( flag_calc ) then

      # calculate RMSEs
      a_rmses = read_RMSEs( out_rmse_path )
      for i in 0..(a_rmses.size-1)
        bra_rmses[iband][iRefProf][i] = a_rmses[i]
      end

      # calculate RMSEs with respect to LBL
      a_rmses = read_RMSEs( out_rmse_lbl_path )
      for i in 0..(a_rmses.size-1)
        bra_rmses_lbl[iband][iRefProf][i] = a_rmses[i]
      end

    else

      for i in 0..(bra_rmses[0].size-1)
        bra_rmses    [iband][iRefProf][i] = 1.0e10
        bra_rmses_lbl[iband][iRefProf][i] = 1.0e10
      end

    end

    #   go back to the parent directory
#    print 'exiting '+dir_work+"\n"
    Dir.chdir('..')

  end

  a_dir_work.each{ |dir_work|
#    print 'deleting working directory, '+dir_work+"\n"
    com = 'rm -rf '+dir_work
    system( com )
  }

  # check minimum RMSEs
  b_rmses = []
  for iband in 0..(nband-1)
    loss = 0.0
    for iRefProf in 0..(nRefProfs-1)
      #       PR.upflx
      loss += bra_rmses[iband][iRefProf][0]+
      #      +SR.upflx
              bra_rmses[iband][iRefProf][1]+
      #      +PR.dnflx
              bra_rmses[iband][iRefProf][2]+
      #      +SR.dnflx
              bra_rmses[iband][iRefProf][3]
    end
    b_rmses << loss
  end
  i_min = b_rmses.index(b_rmses.min)
  print '  Decrease IP in the band #'+(i_min+1).to_s+' (RMSE sum=', b_rmses.min, ')'+"\n"
  # decrease integration point in the (i_min+1)th band
  b_nip[i_min] -= 1
  # check condition
  if ( b_nip[i_min] <= 0 ) then
    print 'Number of integration point in the band #'+(i_min+1).to_s+' will be zero.'+"\n"
    exit
  end

  # output to file
  line = ''
  file = File.open(ipstruct_path, 'a')
  for iband in 0..(nband-1)
    line = iloop.to_s+' '+iband.to_s+' '+b_nip[iband].to_s
    file.puts line
  end
  line = ''
  file.puts line
  file.close
  #
  file = File.open(loss_path, 'a')
  line = iloop.to_s+' '+b_nip.sum.to_s+' '+b_rmses.min.to_s+' '+(i_min+1).to_s
  for iRefProf in 0..(nRefProfs-1)
           #  PR.upflx
    loss = bra_rmses[i_min][iRefProf][0]+
           # +SR.upflx
           bra_rmses[i_min][iRefProf][1]+
           # +PR.dnflx
           bra_rmses[i_min][iRefProf][2]+
           # +SR.dnflx
           bra_rmses[i_min][iRefProf][3]
    line += ' '+loss.to_s
  end
  file.puts line
  file.close


  file = File.open(loss_lbl_path, 'a')
  loss = 0.0
  for iRefProf in 0..(nRefProfs-1)
    loss += bra_rmses_lbl[i_min][iRefProf][0]+
            bra_rmses_lbl[i_min][iRefProf][1]+
            bra_rmses_lbl[i_min][iRefProf][2]+
            bra_rmses_lbl[i_min][iRefProf][3]
  end
  line = iloop.to_s+' '+b_nip.sum.to_s+' '+loss.to_s+' '+(i_min+1).to_s
  for iRefProf in 0..(nRefProfs-1)
           #  PR.upflx
    loss = bra_rmses_lbl[i_min][iRefProf][0]+
           # +SR.upflx
           bra_rmses_lbl[i_min][iRefProf][1]+
           # +PR.dnflx
           bra_rmses_lbl[i_min][iRefProf][2]+
           # +SR.dnflx
           bra_rmses_lbl[i_min][iRefProf][3]
    line += ' '+loss.to_s
  end
  file.puts line
  file.close

end
