program main_ac_process_many

  use vtype_module

  use fi_module

  use ac_process_module, only : ac_process

#ifdef MPI
  use mpi
#endif

  implicit none


  character(extstr)       :: CtlFN  = 'ac_process_many.conf'
  character(extstr)       :: CtlFN2 = 'many.conf'

  integer                 :: iostat
  integer                 :: FileUnit

  character(len=4*extstr) :: InputDirName1
  character(len=4*extstr) :: InputDirName2
  character(len=4*extstr) :: OutputDirName

  character(len=4*extstr) :: InputFileName1
  character(len=4*extstr) :: InputFileName2
  character(len=4*extstr) :: OutputFileName

  character(len=extstr)   :: FileNameTail

  real(dp)                :: Factor1
  real(dp)                :: Factor2

  integer :: iTemp
  integer :: iTemps
  integer :: iTempe
  integer :: Mol1Num
  integer :: iVMRMol1
  integer :: iVMRMol1s
  integer :: iVMRMol1e
  integer :: Mol2Num
  integer :: iVMRMol2
  integer :: iVMRMol2s
  integer :: iVMRMol2e
  integer :: Mol3Num
  integer :: iVMRMol3
  integer :: iVMRMol3s
  integer :: iVMRMol3e

  integer :: NTemp
  integer :: NVMRMol1
  integer :: NVMRMol2
  integer :: NVMRMol3

  integer :: iLoop
  integer :: NLoop
  integer :: iLoops
  integer :: iLoope

  integer :: iRank

  integer :: IErr
  integer :: NProcs
  integer :: MyRank


  namelist /factor_nml/ &
    & Factor1, Factor2
  namelist /dir_nml/ &
    & InputDirName1, InputDirName2, OutputDirName
  namelist /many_nml/ &
    & iTemps, iTempe, &
    & Mol1Num, iVMRMol1s, iVMRMol1e, &
    & Mol2Num, iVMRMol2s, iVMRMol2e, &
    & Mol3Num, iVMRMol3s, iVMRMol3e


#ifdef MPI
  call mpi_init( IErr )
  call mpi_comm_size( mpi_comm_world, NProcs, IErr )
  call mpi_comm_rank( mpi_comm_world, MyRank, IErr )
#else
  NProcs = 1
  MyRank = 0
#endif


  Factor1       = 0.0d0
  Factor2       = 0.0d0
  InputDirName1 = ''
  InputDirName2 = ''
  OutputDirName = ''

  ! Read a namelist file

  call fi_open( CtlFN, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = factor_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in factor_nml is something wrong.'
  end if
  write( 6, nml = factor_nml )
  !
  rewind( FileUnit )
  read( FileUnit, nml = dir_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in dir_nml is something wrong.'
  end if
  write( 6, nml = dir_nml )
  !
  close( FileUnit )


  call fi_open( CtlFN2, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = many_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in many_nml is something wrong.'
  end if
  write( 6, nml = many_nml )
  !
  close( FileUnit )


#ifdef MPI
#else
  if ( NProcsMPISubLoop /= 1 ) then
    write( 6, * ) 'Unexpected NProcsMPISubLoop: ', NProcsMPISubLoop
    stop
  end if
#endif


  NTemp    = iTempe    - iTemps    + 1
  NVMRMol1 = iVMRMol1e - iVMRMol1s + 1
  NVMRMol2 = iVMRMol2e - iVMRMol2s + 1
  NVMRMol3 = iVMRMol3e - iVMRMol3s + 1

  NLoop = NTemp * NVMRMol1 * NVMRMol2 * NVMRMol3

  iLoope = 0
  do iRank = 0, MyRank
    iLoops = iLoope + 1
    iLoope = iLoops + NLoop / NProcs - 1
    if ( iRank < mod( NLoop, NProcs ) ) then
      iLoope = iLoope + 1
    end if
  end do
  ! check
  if ( iLoope > NLoop ) then
    write( 6, * ) 'Unexpected number of maximum of loop: ', iLoope, NLoop
  end if


  do iLoop = iLoops, iLoope

    iTemp       = iTemps       &
      & + ( iLoop - 1 ) / ( NVMRMol1 * NVMRMol2 * NVMRMol3 )
    iVMRMol1 = iVMRMol1s &
      & + mod( ( iLoop - 1 ) / ( NVMRMol2 * NVMRMol3 ), NVMRMol1 )
    iVMRMol2 = iVMRMol2s &
      & + mod( ( iLoop - 1 ) / ( NVMRMol3 ), NVMRMol2 )
    iVMRMol3 = iVMRMol3s &
      & + mod( ( iLoop - 1 ), NVMRMol3 )

    write( FileNameTail, '(a,i3.3,3(a,i2.2,a,i3.3),a)' ) &
      & "T-", iTemp, &
      & "_M", Mol1Num, "-", iVMRMol1, &
      & "_M", Mol2Num, "-", iVMRMol2, &
      & "_M", Mol3Num, "-", iVMRMol3, &
      & ".nc"


    InputFileName1 = trim( InputDirName1 ) // '/ac_' // trim( FileNameTail )
    InputFileName2 = trim( InputDirName2 ) // '/ac_' // trim( FileNameTail )
    OutputFileName = trim( OutputDirName ) // '/ac_' // trim( FileNameTail )

    write( 6, * ) 'Input1: ', trim( InputFileName1 )
    write( 6, * ) 'Input2: ', trim( InputFileName2 )
    write( 6, * ) 'Output: ', trim( OutputFileName )


    call ac_process(                              &
      & InputFileName1, InputFileName2, OutputFileName, &
      & Factor1, Factor2                                &
      & )

  end do


#ifdef MPI
  call mpi_finalize( IErr )
#endif


  write( 6, * ) 'Finish'
end program main_ac_process_many
