! Roland Span, Eric W. Lemmon, Richard T Jacobsen, Wolfgang Wagner,
! and Akimichi Yokozeki,
! A Reference Equation of State for the Thermodynamic Properties of Nitrogen
! for Temperatures from 63.151 to 1000 K and Pressures to 2200 MPa,
! Journal of Physical and Chemical Reference Data, 29, 1361 (2000);
! https://doi.org/10.1063/1.1349047
module thermo_prop_n2_s2000

  use vtype_module

  implicit none

  private

  character(stdstr), parameter :: ModuleName = 'thermo_prop_n2_s2000'

  logical , save :: FlagIdealGas = .false.
  real(dp), save :: ConvThreshold = 1.0d-11

  data FlagIdealGas  /.false./
  data ConvThreshold /1.0d-11/

  real(dp), parameter :: UnivGasConst = 8.314510d0  ! J mole-1 K-1
  real(dp), parameter :: MolWeight    = 28.01348d-3 ! kg mole-1
  real(dp), parameter :: GasConst     = UnivGasConst / MolWeight ! J kg-1 K-1

  real(dp), parameter :: DensC  = 11.1839d0 * MolWeight * 1.0d3
                         ! rho_c (kg m-3) = 11.1839 mol/dm3
  real(dp), parameter :: TempC  = 126.192d0  ! T_c (K)
  real(dp), parameter :: PressC = 3.3958d6   ! p_c (Pa)
  real(dp), parameter :: TempT  = 63.151d0   ! T_t (K)
  real(dp), parameter :: PressT = 12.523d3   ! p_t (Pa)


  real(dp), save :: a_IGA       ( 1: 8)
  real(dp), save :: a_ResCapN   ( 1:36)
  integer , save :: a_ResSmallI ( 1:36)
  real(dp), save :: a_ResSmallJ ( 1:36)
  integer , save :: a_ResSmallL ( 1:36)
  integer , save :: a_ResPhi    (33:36)
  integer , save :: a_ResBeta   (33:36)
  real(dp), save :: a_ResGamma  (33:36)


  ! p.1382 below Eq. (53)
  data a_IGA &
    & / &
    &  2.5d0      , -12.76952708d0, -0.00784163d0, -1.934819d-4, &
    & -1.247742d-5,   6.678326d-8 ,  1.012941d0  , 26.65788d0    &
    & /

  ! TABLE 17 N_k
  data a_ResCapN &
    & / &
    &  0.924803575275d0 , -0.492448489428d0 ,  0.661883336938d0 , &!  1-
    & -0.192902649201d1 , -0.622469309629d-1,  0.349943957581d0 , &!  4-
    &  0.564857472498d0 , -0.161720005987d1 , -0.481395031883d0 , &!  7-
    &  0.421150636384d0 , -0.161962230825d-1,  0.172100994165d0 , &! 10-
    &  0.735448924933d-2,  0.168077305479d-1, -0.107626664179d-2, &! 13-
    & -0.137318088513d-1,  0.635466899859d-3,  0.304432279419d-2, &! 16-
    & -0.435762336045d-1, -0.723174889316d-1,  0.389644315272d-1, &! 19-
    & -0.212201363910d-1,  0.408822981509d-2, -0.551990017984d-4, &! 22-
    & -0.462016716479d-1, -0.300311716011d-2,  0.368825891208d-1, &! 25-
    & -0.255856846220d-2,  0.896915264558d-2, -0.441513370350d-2, &! 28-
    &  0.133722924858d-2,  0.264832491957d-3,  0.196688194015d2 , &! 31-
    & -0.209115600730d2 ,  0.167788306989d-1,  0.262767566274d4   &! 34-
    & /

  ! TABLE 17 i_k
  data a_ResSmallI &
    & /  &
    &  1,  1,  2,  2,  3,  &!  1-
    &  3,  1,  1,  1,  3,  &!  6-
    &  3,  4,  6,  6,  7,  &! 11-
    &  7,  8,  8,  1,  2,  &! 16-
    &  3,  4,  5,  8,  4,  &! 21-
    &  5,  5,  8,  3,  5,  &! 26-
    &  6,  9,  1,  1,  3,  &! 31-
    &  2                   &! 36-
    & /
  ! TABLE 17 j_k
  data a_ResSmallJ &
    & /  &
    &  0.25,  0.875,  0.5 ,  0.875,  0.375, &!  1-
    &  0.75,  0.5  ,  0.75,  2.0  ,  1.25 , &!  6-
    &  3.5 ,  1.0  ,  0.5 ,  3.0  ,  0.0  , &! 11-
    &  2.75,  0.75 ,  2.5 ,  4.0  ,  6.0  , &! 16-
    &  6.0 ,  3.0  ,  3.0 ,  6.0  , 16.0  , &! 21-
    & 11.0 , 15.0  , 12.0 , 12.0  ,  7.0  , &! 26-
    &  4.0 , 16.0  ,  0.0 ,  1.0  ,  2.0  , &! 31-
    &  3.0                                  &! 36-
    & /
  ! TABLE 17 l_k
  data a_ResSmallL &
    & /  &
    &  0, 0, 0, 0, 0, 0, 1, 1, 1, &!  1-
    &  1, 1, 1, 1, 1, 1, 1, 1, 1, &!  9-
    &  2, 2, 2, 2, 2, 2, 3, 3, 3, &! 19-
    &  3, 4, 4, 4, 4, 2, 2, 2, 2  &! 28-
    & /
  ! TABLE 18 phi_k
  data a_ResPhi &
    & /  &
    &  20, 20, 15, 25 &! 33-
    & /
  ! TABLE 18 beta_k
  data a_ResBeta &
    & /  &
    &  325, 325, 300, 275 &! 33-
    & /
  ! TABLE 18 gamma_k
  data a_ResGamma &
    & /  &
    &  1.16, 1.16, 1.13, 1.25 &! 33-
    & /


  public :: ThermoPropInit
  public :: ThermoPropInqUnivGasConst
  public :: ThermoPropInqMolWeight
  public :: ThermoPropInqTempCrit
  public :: ThermoPropInqDensCrit
  public :: ThermoPropInqTempTrip
  public :: ThermoPropHelmholtzEne
  public :: ThermoPropCompFact
  public :: ThermoPropDens
  public :: ThermoPropTemp
  public :: ThermoPropTempFromPressEnthalpy
  public :: ThermoPropEntropy
  public :: ThermoPropEnthalpy
  public :: ThermoPropInternalEnergy
  public :: ThermoPropCp
  public :: ThermoPropCv
  public :: ThermoPropDPressDDensConstTemp
  public :: ThermoPropDPressDTempConstDens
  public :: ThermoPropThermoExpCoef
  public :: ThermoPropAdLapRateDTDP
  public :: ThermoPropDensTempFromPressEnthalpy
  public :: ThermoPropChkSaturated
  !
  public :: IGAlpha
  public :: IGDAlphaDDelta
  public :: IGD2AlphaDDelta2
  public :: IGDAlphaDTau
  public :: IGD2AlphaDTau2
  public :: IGD2AlphaDDeltaDTau
  public :: ResAlpha
  public :: ResDAlphaDDelta
  public :: ResD2AlphaDDelta2
  public :: ResDAlphaDTau
  public :: ResD2AlphaDTau2
  public :: ResD2AlphaDDeltaDTau
  ! subroutines
  public :: ThermoPropMeltPressure
  public :: ThermoPropSublPressure
!!$  public :: ThermoPropSublTemperature
  public :: ThermoPropSatPressure
  public :: ThermoPropJudgePhase

  !----------------------------------------------------------------------------

contains

  !----------------------------------------------------------------------------
  ! set parameters
  !----------------------------------------------------------------------------

  subroutine ThermoPropInit( &
    & FlagIdealGasSet, ConvThresholdSet  &
    & )

    logical , intent(in), optional :: FlagIdealGasSet
    real(dp), intent(in), optional :: ConvThresholdSet


    if ( present( FlagIdealGasSet ) ) then
      FlagIdealGas = FlagIdealGasSet
    end if

    if ( present( ConvThresholdSet ) ) then
      ConvThreshold = ConvThresholdSet
    end if


  end subroutine ThermoPropInit

  !----------------------------------------------------------------------------
  ! inquire molar gas constant
  !----------------------------------------------------------------------------

  function ThermoPropInqUnivGasConst() &
    & result( Val )

    real(dp) :: Val

    Val = UnivGasConst

  end function ThermoPropInqUnivGasConst

  !----------------------------------------------------------------------------
  ! inquire molecular weight
  !----------------------------------------------------------------------------

  function ThermoPropInqMolWeight() &
    & result( Val )

    real(dp) :: Val

    Val = MolWeight

  end function ThermoPropInqMolWeight

  !----------------------------------------------------------------------------
  ! inquire temperature of critical point
  !----------------------------------------------------------------------------

  function ThermoPropInqTempCrit() &
    & result( Val )

    real(dp) :: Val

    Val = TempC

  end function ThermoPropInqTempCrit

  !----------------------------------------------------------------------------
  ! inquire density of critical point
  !----------------------------------------------------------------------------

  function ThermoPropInqDensCrit() &
    & result( Val )

    real(dp) :: Val

    Val = DensC

  end function ThermoPropInqDensCrit

  !----------------------------------------------------------------------------
  ! inquire temperature of triple point
  !----------------------------------------------------------------------------

  function ThermoPropInqTempTrip() &
    & result( Val )

    real(dp) :: Val

    Val = TempT

  end function ThermoPropInqTempTrip

  !----------------------------------------------------------------------------
  ! Helmholtz energy (Eq. (10))
  !----------------------------------------------------------------------------

  function ThermoPropHelmholtzEne( Dens, Temp ) &
    & result( HEne )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: HEne


    ! Local variables
    !
    real(dp) :: Delta
    real(dp) :: Tau
    real(dp) :: HEneIG
    real(dp) :: HEneRes


    Delta = Dens  / DensC
    Tau   = TempC / Temp

    HEneIG  = IGAlpha ( Delta, Tau )
    HEneRes = ResAlpha( Delta, Tau )

    if ( FlagIdealGas ) then
      HEne = HEneIG
    else
      HEne = HEneIG + HEneRes
    end if

    HEne = HEne * GasConst * Temp


  end function ThermoPropHelmholtzEne

  !----------------------------------------------------------------------------
  ! compressibility factor
  !----------------------------------------------------------------------------

  function ThermoPropCompFact( Dens, Temp ) &
    & result( CompFact )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: CompFact


    ! Local variables
    !
    real(dp) :: Delta
    real(dp) :: Tau


    Delta = Dens  / DensC
    Tau   = TempC / Temp

    if ( FlagIdealGas ) then
      CompFact = 1.0d0
    else
      CompFact = 1.0d0 + Delta * ResDAlphaDDelta( Delta, Tau )
    end if


  end function ThermoPropCompFact

  !----------------------------------------------------------------------------
  ! density
  !----------------------------------------------------------------------------

  function ThermoPropDens( Press, Temp ) &
    & result( Dens )

    real(dp), intent(in) :: Press
    real(dp), intent(in) :: Temp
    real(dp)             :: Dens


    ! Local variables
    !
    character(stdstr) :: CPhase

    real(dp) :: DensCur
    real(dp) :: DensNext
    real(dp) :: DPDRhoCT
    real(dp) :: DelDens

    integer, parameter  :: NLoop = 100
    integer             :: iLoop
    real(dp)            :: a_DensHist( 0:NLoop )


    ! Initialization
    call ThermoPropJudgePhase( Temp, Press, CPhase )
    select case ( CPhase )
    case ( 'G', 'C' )
      DensCur = Press / ( GasConst * Temp )
    case default
      DensCur = 1500.0d0
    end select

    a_DensHist(0) = DensCur
    search_pressure : do iLoop = 1, NLoop
      DPDRhoCT = ThermoPropDPressDDensConstTemp( DensCur, Temp )
      DelDens =                                                &
        &   (   Press                                          &
        &     - ThermoPropCompFact( DensCur, Temp )   &
        &       * DensCur * GasConst * Temp                  ) &
        & / DPDRhoCT
      DensNext = DensCur + DelDens
      a_DensHist( iLoop ) = DensNext
!!$      write( 6, * ) iLoop, DensCur, DensNext
      if ( abs( ( DensNext - DensCur ) / DensCur ) < ConvThreshold ) then
        exit search_pressure
      end if
      DensCur  = DensNext
    end do search_pressure
    if ( iLoop >= NLoop ) then
      write( 6, * ) 'ERROR: In ', trim( ModuleName )
      write( 6, * ) 'ERROR: In ThermoPropDens'
      write( 6, * ) 'ERROR:   Loop does not converge, ', Press, Temp
      write( 6, * ) 'ERROR:   ', iLoop
      write( 6, * ) 'ERROR:   ', a_DensHist
      stop
    end if

    Dens = DensNext


  end function ThermoPropDens

  !----------------------------------------------------------------------------
  ! temperature
  !----------------------------------------------------------------------------

  function ThermoPropTemp( Press, Dens ) &
    & result( Temp )

    real(dp), intent(in) :: Press
    real(dp), intent(in) :: Dens
    real(dp)             :: Temp


    ! Local variables
    !
    real(dp) :: Temp1
    real(dp) :: Temp2
    real(dp) :: DensCur
    real(dp) :: TempCur
    real(dp) :: TempNext
    real(dp) :: DPDTCRho
    real(dp) :: DelTemp
    integer, parameter  :: NLoop = 100
    integer             :: iLoop
    real(dp)            :: a_Hist( 0:NLoop )


    ! Initialization
!!$    if ( Dens < DensC ) then
!!$      ! MEMO
!!$      !   Facot 0.8 is required to obtain appropriate temperatures
!!$      !   when the state is close to liquid-gas boundary.
!!$      !   If this factor is not included, this routine converges to wrong
!!$      !   temperatures in some cases.
!!$      !
!!$      TempCur = Press / ( Dens * GasConst ) / 0.8d0
!!$    else
!!$      TempCur = 200.0d0
!!$      TempCur = 250.0d0
!!$    end if

    ! Bisectional method
    Temp1   = 50.0d0
    DensCur = ThermoPropDens( Press, Temp1 )
    if ( DensCur < Dens ) then
      write( 6, * ) 'ERROR: In ', trim( ModuleName )
      write( 6, * ) 'ERROR: In ThermoPropTemp'
      write( 6, * ) 'ERROR:   Temperature seems to be too low.'
      write( 6, * ) 'ERROR:   ', Dens, DensCur
      stop
    end if
    search_init_temperature : do
      if ( Temp1 < 100.0d0 ) then
        Temp2 = Temp1 + 10.0d0
      else
        Temp2 = Temp1 + 100.0d0
      end if
      DensCur = ThermoPropDens( Press, Temp2 )
      if ( DensCur < Dens ) then
        exit
      end if
      Temp1 = Temp2
    end do search_init_temperature
    bisectional_method : do
      TempCur = ( Temp1 + Temp2 ) / 2.0d0
      DensCur = ThermoPropDens( Press, TempCur )
      if ( DensCur < Dens ) then
        Temp2 = TempCur
      else
        Temp1 = TempCur
      end if
      if ( abs( ( Temp2 - Temp1 ) / Temp1 ) < ConvThreshold ) then
        exit bisectional_method
      end if
    end do bisectional_method
    TempCur = ( Temp1 + Temp2 ) / 2.0d0


    a_Hist(0) = TempCur
    search_temperature : do iLoop = 1, NLoop
      DPDTCRho = ThermoPropDPressDTempConstDens( Dens, TempCur )
      DelTemp =                                                &
        &   (   Press                                          &
        &     - ThermoPropCompFact( Dens, TempCur )   &
        &       * Dens * GasConst * TempCur                  ) &
        & / DPDTCRho
      TempNext = TempCur + DelTemp
      a_Hist( iLoop ) = TempNext
!!$      write( 6, * ) iLoop, TempCur, TempNext
!!$      write( 6, * ) ThermoPropDens( Press, TempNext )
      if ( abs( ( TempNext - TempCur ) / TempCur ) < ConvThreshold ) then
        exit search_temperature
      end if
      TempCur  = TempNext
    end do search_temperature
    if ( iLoop >= NLoop ) then
      write( 6, * ) 'ERROR: In ', trim( ModuleName )
      write( 6, * ) 'ERROR: In ThermoPropTemp'
      write( 6, * ) 'ERROR:   Loop does not converge'
      write( 6, * ) 'ERROR:   ', iLoop
      write( 6, * ) 'ERROR:   ', a_Hist
      stop
    end if

    Temp = TempNext


  end function ThermoPropTemp

  !----------------------------------------------------------------------------
  ! temperature from pressure and enthalpy
  !
  ! This routine calculate temperature by the use of pressure and enthalpy.
  ! But, this routine requires a very small threshold for convergence,
  ! ConvThreshold ~ 1e-9 or 1e-10 or so.
  ! Instead, a routine using ThermoPropDensTempFromPressEnthalpy seems to
  ! calculate better temperatures with larger threshold, ConvThreshold ~ 1e-5.
  !----------------------------------------------------------------------------

  function NG_ThermoPropTempFromPressEnthalpy( Press, Enthalpy ) & ! (in)
    & result( Temp )

    real(dp), intent(in ) :: Press
    real(dp), intent(in ) :: Enthalpy
    real(dp)              :: Temp


    ! Local variables
    !
!!$    real(dp) :: TempMin = 60.0d0
    real(dp) :: Dens
    real(dp) :: TempCur
!!$    real(dp) :: TempNext
    real(dp) :: Temp1
    real(dp) :: Temp2
    real(dp) :: Enthalpy1
    real(dp) :: Enthalpy2
    real(dp) :: EnthalpyCur
!!$    real(dp) :: DHDTCP
!!$    real(dp) :: DelTemp
!!$    integer, parameter  :: NLoop = 100
!!$    integer             :: iLoop
!!$    real(dp)            :: a_HistDens( 0:NLoop )
!!$    real(dp)            :: a_HistTemp( 0:NLoop )


    ! Initialization
    !   find temperature with a given pressure

    ! Bisectional method
    Temp1     = 50.0d0
    Dens      = ThermoPropDens( Press, Temp1 )
    Enthalpy1 = ThermoPropEnthalpy( Dens, Temp1 )
    if ( Enthalpy < Enthalpy1 ) then
      write( 6, * ) 'ERROR: In ', trim( ModuleName )
      write( 6, * ) 'ERROR: In ThermoPropTempFromPressEnthalpy'
      write( 6, * ) 'ERROR:   Temperature seems to be too low.'
      stop
    end if
    search_init_temperature : do
      if ( Temp1 < 100.0d0 ) then
        Temp2     = Temp1 + 10.0d0
      else
        Temp2     = Temp1 + 100.0d0
      end if
      Dens      = ThermoPropDens( Press, Temp2 )
      Enthalpy2 = ThermoPropEnthalpy( Dens, Temp2 )
      if ( Enthalpy < Enthalpy2 ) then
        exit
      end if
      Temp1 = Temp2
    end do search_init_temperature
    bisectional_method : do
      TempCur     = ( Temp1 + Temp2 ) / 2.0d0
      Dens        = ThermoPropDens( Press, TempCur )
      EnthalpyCur = ThermoPropEnthalpy( Dens, TempCur )
      if ( Enthalpy < EnthalpyCur ) then
        Temp2 = TempCur
      else
        Temp1 = TempCur
      end if
      if ( abs( ( Temp2 - Temp1 ) / Temp1 ) < ConvThreshold ) then
        exit bisectional_method
      end if
    end do bisectional_method
    Temp = ( Temp1 + Temp2 ) / 2.0d0


    ! Newton's method
!!$    TempCur = TempMin
!!$    a_HistTemp(0) = TempCur
!!$    search_temperature : do iLoop = 1, NLoop
!!$      Dens        = ThermoPropDens( Press, TempCur )
!!$      EnthalpyCur = ThermoPropEnthalpy( Dens, TempCur )
!!$      DHDTCP      = ThermoPropCp( Dens, TempCur )
!!$      if ( DHDTCP <= 0.0d0 ) then
!!$        write( 6, * ) 'ERROR: In ', trim( ModuleName )
!!$        write( 6, * ) 'ERROR: In ThermoPropTempFromPressEnthalpy'
!!$        write( 6, * ) 'ERROR:   (DHDT)_p is negative, ', DHDTCP
!!$        write( 6, * ) 'ERROR:   at rho = ', Dens, ', T = ', TempCur
!!$        stop
!!$      end if
!!$      DelTemp     = ( Enthalpy - EnthalpyCur ) / DHDTCP
!!$      TempNext    = TempCur + DelTemp
!!$      TempNext    = max( TempNext, TempMin )
!!$      a_HistTemp( iLoop ) = TempNext
!!$      if ( abs( ( TempNext - TempCur ) / TempCur ) < ConvThreshold ) then
!!$        exit search_temperature
!!$      end if
!!$      TempCur = TempNext
!!$    end do search_temperature
!!$    if ( iLoop >= NLoop ) then
!!$      write( 6, * ) 'ERROR: In ', trim( ModuleName )
!!$      write( 6, * ) 'ERROR: In ThermoPropTempFromPressEnthalpy'
!!$      write( 6, * ) 'ERROR:   Loop for temperature does not converge'
!!$      write( 6, * ) 'ERROR:   ', iLoop
!!$      write( 6, * ) 'ERROR:   ', a_HistTemp
!!$      stop
!!$    end if
!!$    Temp = TempNext
!!$
!!$    if ( Temp == TempMin ) then
!!$      write( 6, * ) 'ERROR: In ', trim( ModuleName )
!!$      write( 6, * ) 'ERROR: In ThermoPropTempFromPressEnthalpy'
!!$      write( 6, * ) 'ERROR:   Temperature seems to be calculated, properly.'
!!$      write( 6, * ) 'ERROR:   ', Temp, TempMin
!!$      stop
!!$    end if


  end function NG_ThermoPropTempFromPressEnthalpy

  !----------------------------------------------------------------------------
  ! temperature from pressure and enthalpy
  !----------------------------------------------------------------------------

  function ThermoPropTempFromPressEnthalpy( Press, Enthalpy ) & ! (in)
    & result( Temp )

    real(dp), intent(in ) :: Press
    real(dp), intent(in ) :: Enthalpy
    real(dp)              :: Temp


    ! Local variables
    !
    real(dp) :: Dens


    call ThermoPropDensTempFromPressEnthalpy( &
      & Press, Enthalpy, & ! (in)
      & Dens, Temp       & ! (out)
      & )


  end function ThermoPropTempFromPressEnthalpy

  !----------------------------------------------------------------------------
  ! entropy (Eq. (60))
  !----------------------------------------------------------------------------

  function ThermoPropEntropy( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !
    real(dp) :: Delta
    real(dp) :: Tau


    Delta = Dens  / DensC
    Tau   = TempC / Temp

    if ( FlagIdealGas ) then
      Val =                           &
        &   Tau * IGDAlphaDTau( Tau ) &
        & - IGAlpha( Delta, Tau )
    else
      Val =                                                             &
        &   Tau * ( IGDAlphaDTau( Tau ) + ResDAlphaDTau( Delta, Tau ) ) &
        & - ( IGAlpha( Delta, Tau ) + ResAlpha( Delta, Tau ) )
    end if

    Val = Val * GasConst


  end function ThermoPropEntropy

  !----------------------------------------------------------------------------
  ! enthalpy (Eq. (59))
  !----------------------------------------------------------------------------

  function ThermoPropEnthalpy( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !
    real(dp) :: Delta
    real(dp) :: Tau


    Delta = Dens  / DensC
    Tau   = TempC / Temp

    if ( FlagIdealGas ) then
      Val =                                                             &
        &   1.0d0                                                       &
        & + Tau * IGDAlphaDTau( Tau )
    else
      Val =                                                             &
        &   1.0d0                                                       &
        & + Tau * ( IGDAlphaDTau( Tau ) + ResDAlphaDTau( Delta, Tau ) ) &
        & + Delta * ResDAlphaDDelta( Delta, Tau )
    end if

    Val = Val * GasConst * Temp


  end function ThermoPropEnthalpy

  !----------------------------------------------------------------------------
  ! internal energy (Eq. (58))
  !----------------------------------------------------------------------------

  function ThermoPropInternalEnergy( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !
    real(dp) :: Delta
    real(dp) :: Tau


    Delta = Dens  / DensC
    Tau   = TempC / Temp

    if ( FlagIdealGas ) then
      Val =                                                         &
        &   Tau * IGDAlphaDTau( Tau )
    else
      Val =                                                         &
        & + Tau * ( IGDAlphaDTau( Tau ) + ResDAlphaDTau( Delta, Tau ) )
    end if

    Val = Val * GasConst * Temp


  end function ThermoPropInternalEnergy

  !----------------------------------------------------------------------------
  ! isobaric heat capacity (specific heat on constant pressure) (Eq. (63))
  !----------------------------------------------------------------------------

  function ThermoPropCp( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !
    real(dp) :: Delta
    real(dp) :: Tau


    Delta = Dens  / DensC
    Tau   = TempC / Temp

    if ( FlagIdealGas ) then
      Val =                                                             &
        &   1.0d0
    else
      Val =                                                             &
        &     (   1.0d0                                                 &
        &       + Delta * ResDAlphaDDelta( Delta, Tau )                 &
        &       - Delta * Tau * ResD2AlphaDDeltaDTau( Delta, Tau ) )**2 &
        &   / (   1.0d0                                                 &
        &       + 2.0d0 * Delta * ResDAlphaDDelta( Delta, Tau )         &
        &       + Delta**2 * ResD2AlphaDDelta2( Delta, Tau ) )
    end if

    Val = Val * GasConst

    Val = Val + ThermoPropCv( Dens, Temp )


  end function ThermoPropCp

  !----------------------------------------------------------------------------
  ! isochoric heat capacity (specific heat on constant volume) (Eq. (62))
  !----------------------------------------------------------------------------

  function ThermoPropCv( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !
    real(dp) :: Delta
    real(dp) :: Tau


    Delta = Dens  / DensC
    Tau   = TempC / Temp

    if ( FlagIdealGas ) then
      Val =                                                           &
        & - Tau**2 * IGD2AlphaDTau2( Tau )
    else
      Val =                                                           &
        & - Tau**2 * (   IGD2AlphaDTau2( Tau )                        &
        &              + ResD2AlphaDTau2( Delta, Tau ) )
    end if

    Val = Val * GasConst


  end function ThermoPropCv

  !----------------------------------------------------------------------------
  ! DPDRho (Eq. (68))
  !----------------------------------------------------------------------------

  function ThermoPropDPressDDensConstTemp( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !
    real(dp) :: Delta
    real(dp) :: Tau


    Delta = Dens  / DensC
    Tau   = TempC / Temp

    if ( FlagIdealGas ) then
      Val = GasConst * Temp * Delta                         &
        & * (   2.0d0 * IGDAlphaDDelta   ( Delta )          &
        &     + Delta * IGD2AlphaDDelta2 ( Delta ) )
    else
      Val = GasConst * Temp * Delta                             &
        & * (   2.0d0 * (   IGDAlphaDDelta   ( Delta )          &
        &                 + ResDAlphaDDelta  ( Delta, Tau ) )   &
        &     + Delta * (   IGD2AlphaDDelta2 ( Delta )          &
        &                 + ResD2AlphaDDelta2( Delta, Tau ) ) )
    end if


  end function ThermoPropDPressDDensConstTemp

  !----------------------------------------------------------------------------
  ! DPDT (Eq. (70))
  !----------------------------------------------------------------------------

  function ThermoPropDPressDTempConstDens( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !
    real(dp) :: Delta
    real(dp) :: Tau


    Delta = Dens / DensC
    Tau   = TempC / Temp

    if ( FlagIdealGas ) then
      Val = Dens * GasConst * Delta                            &
        & * (   IGDAlphaDDelta   ( Delta )                     &
        &     - Tau * IGD2AlphaDDeltaDTau () )
    else
      Val = Dens * GasConst * Delta                              &
        & * (   IGDAlphaDDelta   ( Delta )                       &
        &     + ResDAlphaDDelta  ( Delta, Tau )                  &
        &     - Tau * (   IGD2AlphaDDeltaDTau ()                 &
        &               + ResD2AlphaDDeltaDTau( Delta, Tau ) ) )
    end if


  end function ThermoPropDPressDTempConstDens

  !----------------------------------------------------------------------------
  ! DHDRho
  !----------------------------------------------------------------------------

  function ThermoPropDEnthalpyDDensConstTemp( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !
    real(dp) :: Delta
    real(dp) :: Tau


    Delta = Dens  / DensC
    Tau   = TempC / Temp

    if ( FlagIdealGas ) then
      Val =                                                                   &
        & GasConst * TempC / ( DensC * Tau )                                  &
        & * Tau * IGD2AlphaDDeltaDTau()
    else
      Val =                                                                          &
        & GasConst * TempC / ( DensC * Tau )                                         &
        & * (   Tau * ( IGD2AlphaDDeltaDTau() + ResD2AlphaDDeltaDTau( Delta, Tau ) ) &
        &     + ResDAlphaDDelta( Delta, Tau ) + Delta * ResD2AlphaDDelta2( Delta, Tau ) )
    end if


  end function ThermoPropDEnthalpyDDensConstTemp

  !----------------------------------------------------------------------------
  ! DHDT
  !----------------------------------------------------------------------------

  function ThermoPropDEnthalpyDTempConstDens( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !
    real(dp) :: Delta
    real(dp) :: Tau


    Delta = Dens  / DensC
    Tau   = TempC / Temp

    if ( FlagIdealGas ) then
      Val =                                                       &
        & GasConst                                                &
        & * (   1.0d0                                             &
        &     - Tau**2 * IGD2AlphaDTau2( Tau ) )
    else
      Val =                                                       &
        & GasConst                                                &
        & * (   1.0d0                                             &
        &     + Delta * ResDAlphaDDelta( Delta, Tau )             &
        &     - Tau**2 * ( IGD2AlphaDTau2( Tau ) + ResD2AlphaDTau2( Delta, Tau ) ) &
        &     - Delta * Tau * ResD2AlphaDDeltaDTau( Delta, Tau ) )
    end if


  end function ThermoPropDEnthalpyDTempConstDens

  !----------------------------------------------------------------------------
  ! thermal expansion coefficient (Eq. (74))
  !----------------------------------------------------------------------------

  function ThermoPropThermoExpCoef( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !


    Val = &
      &   ThermoPropDPressDTempConstDens( Dens, Temp ) &
      & / ( ThermoPropDPressDDensConstTemp( Dens, Temp ) * Dens )


  end function ThermoPropThermoExpCoef

  !----------------------------------------------------------------------------
  ! adiabatic lapse rate
  !----------------------------------------------------------------------------

  function ThermoPropAdLapRateDTDP( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !
    real(dp) :: Cp
    real(dp) :: ExpCoef


    Cp       = ThermoPropCp           ( Dens, Temp )
    ExpCoef  = ThermoPropThermoExpCoef( Dens, Temp )

    Val = Temp / ( Cp * Dens ) * ExpCoef


  end function ThermoPropAdLapRateDTDP

  !----------------------------------------------------------------------------
  ! temperature and density from pressure and enthalpy
  !----------------------------------------------------------------------------

  subroutine ThermoPropDensTempFromPressEnthalpy( &
    & Press, Enthalpy, & ! (in)
    & Dens, Temp       & ! (out)
    & )

    real(dp), intent(in ) :: Press
    real(dp), intent(in ) :: Enthalpy
    real(dp), intent(out) :: Dens
    real(dp), intent(out) :: Temp


    ! Local variables
    !
    real(dp) :: Temp1
    real(dp) :: Temp2
!!$    real(dp) :: Dens1
!!$    real(dp) :: Dens2
    real(dp) :: Enthalpy1
    real(dp) :: Enthalpy2

    real(dp) :: Delta
    real(dp) :: Tau
    real(dp) :: DensCur
    real(dp) :: DensNext
    real(dp) :: TempCur
    real(dp) :: TempNext
    real(dp) :: PressCur
    real(dp) :: EnthalpyCur
    real(dp) :: DPDRhoCT
    real(dp) :: DPDTCRho
    real(dp) :: DHDRhoCT
    real(dp) :: DHDTCRho
    real(dp) :: DelDens
    real(dp) :: DelTemp
    integer, parameter  :: NLoop = 100
    integer             :: iLoop
    real(dp)            :: a_HistDens( 0:NLoop )
    real(dp)            :: a_HistTemp( 0:NLoop )


    ! Initialization
    !   find temperature with a given pressure

    ! Bisectional method
    Temp1     = 50.0d0
    DensCur   = ThermoPropDens( Press, Temp1 )
    Enthalpy1 = ThermoPropEnthalpy( DensCur, Temp1 )
    if ( Enthalpy < Enthalpy1 ) then
      write( 6, * ) 'ERROR: In ', trim( ModuleName )
      write( 6, * ) 'ERROR: In ThermoPropDensTempFromPressEnthalpy'
      write( 6, * ) 'ERROR:   Temperature seems to be too low.'
      stop
    end if
    search_init_temperature : do
      if ( Temp1 < 100.0d0 ) then
        Temp2     = Temp1 + 10.0d0
      else
        Temp2     = Temp1 + 100.0d0
      end if
      DensCur   = ThermoPropDens( Press, Temp2 )
      Enthalpy2 = ThermoPropEnthalpy( DensCur, Temp2 )
      if ( Enthalpy < Enthalpy2 ) then
        exit
      end if
      Temp1 = Temp2
    end do search_init_temperature
    bisectional_method : do
      TempCur     = ( Temp1 + Temp2 ) / 2.0d0
      DensCur     = ThermoPropDens( Press, TempCur )
      EnthalpyCur = ThermoPropEnthalpy( DensCur, TempCur )
      if ( Enthalpy < EnthalpyCur ) then
        Temp2 = TempCur
      else
        Temp1 = TempCur
      end if
      if ( abs( ( Temp2 - Temp1 ) / Temp1 ) < ConvThreshold ) then
        exit bisectional_method
      end if
    end do bisectional_method
    TempCur = ( Temp1 + Temp2 ) / 2.0d0


    ! Newton's method
    ! NOTE: This does not work well in some cases, i.e., case of liquid.
!!$    TempCur = 300.0d0
!!$    a_HistTemp(0) = TempCur
!!$    search_temperature : do iLoop = 1, NLoop
!!$      Temp1     = TempCur
!!$      Temp2     = Temp1 + 1.0d-1
!!$      Dens1     = ThermoPropDens( Press, Temp1 )
!!$      Dens2     = ThermoPropDens( Press, Temp2 )
!!$      Enthalpy1 = ThermoPropEnthalpy( Dens1, Temp1 )
!!$      Enthalpy2 = ThermoPropEnthalpy( Dens2, Temp2 )
!!$      ! (DHDT)_p
!!$      DHDT = ( Enthalpy2 - Enthalpy1 ) / ( Temp2 - Temp1 )
!!$      !
!!$      EnthalpyCur = Enthalpy1
!!$      DelTemp = ( Enthalpy - EnthalpyCur ) / DHDT
!!$      TempNext = TempCur + DelTemp
!!$      TempNext = max( TempNext, 50.0d0 )
!!$      a_HistTemp( iLoop ) = TempNext
!!$      write( 6, * ) iLoop, TempCur, TempNext
!!$      if ( abs( ( TempNext - TempCur ) / TempCur ) < ConvThreshold ) then
!!$        exit search_temperature
!!$      end if
!!$      TempCur = TempNext
!!$    end do search_temperature
!!$    if ( iLoop >= NLoop ) then
!!$      write( 6, * ) 'ERROR: In ', trim( ModuleName )
!!$      write( 6, * ) 'ERROR: In ThermoPropDensTempFromPressEnthalpy'
!!$      write( 6, * ) 'ERROR:   Loop for temperature does not converge'
!!$      write( 6, * ) 'ERROR:   ', iLoop
!!$      write( 6, * ) 'ERROR:   ', a_HistTemp
!!$      stop
!!$    end if
!!$    TempCur = TempNext


    DensCur = ThermoPropDens( Press, TempCur )

    a_HistDens(0) = DensCur
    a_HistTemp(0) = TempCur
    search_density_temperature : do iLoop = 1, NLoop
      Delta = DensCur / DensC
      Tau   = TempC / TempCur

      PressCur    =                                       &
        & ThermoPropCompFact( DensCur, TempCur ) &
        & * DensCur * GasConst * TempCur
      EnthalpyCur = ThermoPropEnthalpy( DensCur, TempCur )

      DPDRhoCT = ThermoPropDPressDDensConstTemp( DensCur, TempCur )
      DPDTCRho = ThermoPropDPressDTempConstDens( DensCur, TempCur )

      if ( FlagIdealGas ) then
!!$        DHDRhoCT =                                                              &
!!$          & GasConst * TempC / ( DensC * Tau )                                &
!!$          & * Tau * IGD2PhiDDeltaDTau()
        DHDRhoCT = 0.0d0
        DHDTCRho =                                                  &
          & GasConst                                                &
          & * (   1.0d0                                             &
          &     - Tau**2 * IGD2AlphaDTau2( Tau ) )
        DelTemp =                                         &
          &   (   ( Enthalpy - EnthalpyCur ) * DPDRhoCT   &
          &     - ( Press    - PressCur    ) * DHDRhoCT ) &
          & / ( DHDTCRho * DPDRhoCT - DHDRhoCT * DPDTCRho )
        TempNext = TempCur + DelTemp
        DelDens = Press / ( GasConst * TempNext ) - DensCur
        DensNext = DensCur + DelDens
      else
        DHDRhoCT =                                                                 &
          & GasConst * TempC / ( DensC * Tau )                                     &
          & * (   Tau * ( IGD2AlphaDDeltaDTau() + ResD2AlphaDDeltaDTau( Delta, Tau ) ) &
          &     + ResDAlphaDDelta( Delta, Tau ) + Delta * ResD2AlphaDDelta2( Delta, Tau ) )
        DHDTCRho =                                                  &
          & GasConst                                                &
          & * (   1.0d0                                             &
          &     + Delta * ResDAlphaDDelta( Delta, Tau )               &
          &     - Tau**2 * ( IGD2AlphaDTau2( Tau ) + ResD2AlphaDTau2( Delta, Tau ) ) &
          &     - Delta * Tau * ResD2AlphaDDeltaDTau( Delta, Tau ) )
        DelTemp =                                         &
          &   (   ( Enthalpy - EnthalpyCur ) * DPDRhoCT   &
          &     - ( Press    - PressCur    ) * DHDRhoCT ) &
          & / ( DHDTCRho * DPDRhoCT - DHDRhoCT * DPDTCRho )
        DelDens = &
          & ( Enthalpy - EnthalpyCur - DHDTCRho * DelTemp ) / DHDRhoCT
        DensNext = DensCur + DelDens
        TempNext = TempCur + DelTemp
      end if

      a_HistDens( iLoop ) = DensNext
      a_HistTemp( iLoop ) = TempNext
!!$      write( 6, * ) iLoop, DensCur, DensNext, TempCur, TempNext
      if ( ( abs( ( DensNext - DensCur ) / DensCur ) < ConvThreshold ) .and. &
        &  ( abs( ( TempNext - TempCur ) / TempCur ) < ConvThreshold ) ) then
        exit search_density_temperature
      end if
      DensCur  = DensNext
      TempCur  = TempNext
    end do search_density_temperature
    if ( iLoop >= NLoop ) then
      write( 6, * ) 'ERROR: In ', trim( ModuleName )
      write( 6, * ) 'ERROR: In ThermoPropDensTempFromPressEnthalpy'
      write( 6, * ) 'ERROR:   Loop does not converge'
      write( 6, * ) 'ERROR:   ', iLoop
      write( 6, * ) 'ERROR:   ', a_HistDens
      write( 6, * ) 'ERROR:   ', a_HistTemp
      stop
    end if

    Dens = DensNext
    Temp = TempNext


  end subroutine ThermoPropDensTempFromPressEnthalpy

  !----------------------------------------------------------------------------
  ! check saturation density, Eq. (3.15)
  !----------------------------------------------------------------------------

  function ThermoPropChkSaturated( Dens, Temp ) &
    & result( Flag )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    logical              :: Flag


    ! Local variables
    !
    real(dp) :: SatDens


    if ( Temp > TempC ) then
      Flag = .false.
    else
      SatDens = ThermoPropSatVapDensity( Temp )
      if ( Dens <= SatDens ) then
        Flag = .false.
      else
        Flag = .true.
      end if
    end if


  end function ThermoPropChkSaturated

  !----------------------------------------------------------------------------
  ! saturation density, (Eq. (6))
  !----------------------------------------------------------------------------

  function ThermoPropSatVapDensity( Temp ) &
    & result( SatDens )

    real(dp), intent(in) :: Temp
    real(dp)             :: SatDens


    ! Local variables
    !
    integer, parameter :: NCoef = 5
    real(dp)           :: a_SatDensCoef( NCoef )
    real(dp)           :: a_SatDensExp ( NCoef )

    real(dp)           :: TempNorm

    integer            :: i

    data a_SatDensCoef / -1.70127164, -3.70402649, 1.29859383, -0.561424977, -2.68505381 /
!!$    data a_SatDensExp  / 0.340, 5.0/6.0, 7.0/6.0, 13.0/6.0, 14.0/3.0 /
    data a_SatDensExp  / 0.340, 0.833, 1.167, 2.167, 4.667 /


    TempNorm = Temp / TempC

    if ( TempNorm > 1.0d0 ) then
      write( 6, * ) 'ERROR: In ', trim( ModuleName )
      write( 6, * ) 'ERROR: In ThermoPropSatVapDensity'
      write( 6, * ) 'ERROR: Temperature is too high. ', Temp
      stop
    end if


    SatDens = 0.0d0
    do i = 1, NCoef
      SatDens = SatDens &
        & + a_SatDensCoef(i) * ( 1.0d0 - TempNorm )**a_SatDensExp(i)
    end do
    SatDens = SatDens / TempNorm
    SatDens = DensC * exp( SatDens )


  end function ThermoPropSatVapDensity

  !----------------------------------------------------------------------------
  ! judge phase
  !----------------------------------------------------------------------------

  subroutine ThermoPropJudgePhase( Temp, Press, CPhase )

    real(dp)    , intent(in ) :: Temp
    real(dp)    , intent(in ) :: Press
    character(*), intent(out) :: CPhase


    ! Local variables
    !
    real(dp) :: PressBoundary


    if ( Temp > TempC ) then
      if ( Press <= PressC ) then
        CPhase = 'G'
      else
        CPhase = 'C'
      end if
    else
      if ( Temp > TempT ) then
        call ThermoPropSatPressure ( Temp, PressBoundary )
        if ( Press <= PressBoundary ) then
          CPhase = 'G'
        else
          call ThermoPropMeltPressure( Temp, PressBoundary )
          if ( Press <= PressBoundary ) then
            CPhase = 'L'
          else
            CPhase = 'S'
          end if
        end if
      else
        call ThermoPropSublPressure( Temp, PressBoundary )
        if ( Press <= PressBoundary ) then
          CPhase = 'G'
        else
          CPhase = 'S'
        end if
      end if
    end if


  end subroutine ThermoPropJudgePhase

  !----------------------------------------------------------------------------
  ! melting pressure, Eq. (7)
  !   boundary between solid and liquid
  !----------------------------------------------------------------------------

  subroutine ThermoPropMeltPressure( Temp, Val )

    real(dp), intent(in ) :: Temp
    real(dp), intent(out) :: Val


    ! Local variables
    !
    real(dp)            :: TempNorm


    TempNorm = Temp / TempT

    if ( TempNorm < 1.0d0 ) then
      write( 6, * ) 'ERROR: In ', trim( ModuleName )
      write( 6, * ) 'ERROR: In ThermoPropMeltPressure'
      write( 6, * ) 'ERROR: Temperature is too low. ', Temp
      stop
    end if


    Val = ( 12798.61d0 * ( TempNorm**1.78963 - 1.0d0 ) + 1.0d0 ) &
      & * PressT


  end subroutine ThermoPropMeltPressure

  !----------------------------------------------------------------------------
  ! sablimation pressure, Eq. (4)
  !   boundary between solid and gas
  !----------------------------------------------------------------------------

  subroutine ThermoPropSublPressure( Temp, Val )

    real(dp), intent(in ) :: Temp
    real(dp), intent(out) :: Val


    ! Local variables
    !
    real(dp)            :: TempNorm


    call ThermoPropSatPressure( Temp, Val )


  end subroutine ThermoPropSublPressure

  !----------------------------------------------------------------------------

  ! OLD (for CO2)
!!$  subroutine ThermoPropSublTemperature( Press, Val )
!!$
!!$    real(dp), intent(in ) :: Press
!!$    real(dp), intent(out) :: Val
!!$
!!$
!!$    ! Local variables
!!$    !
!!$!    real(dp)            :: TempNorm
!!$!    real(dp)            :: Tmp
!!$!    real(dp)            :: DTmpDT
!!$!    real(dp)            :: DPDT
!!$!    real(dp)            :: TempCur
!!$!    real(dp)            :: TempNext
!!$!    real(dp)            :: PressCur
!!$!    real(dp)            :: DelTemp
!!$!    integer, parameter  :: NLoop = 100
!!$!    integer             :: iLoop
!!$!    real(dp)            :: a_Hist( 0:NLoop )
!!$
!!$    real(dp)            :: Temp1
!!$    real(dp)            :: Temp2
!!$    real(dp)            :: Press1
!!$    real(dp)            :: Press2
!!$    real(dp)            :: TempCur
!!$    real(dp)            :: PressCur
!!$
!!$!    ! Newton's method
!!$
!!$!    ! initialization
!!$!    TempCur = TempT - 1.0d0
!!$!
!!$!    a_Hist(0) = TempCur
!!$!    search_subltemp : do iLoop = 1, NLoop
!!$!      TempNorm = TempCur / TempT
!!$!
!!$!      if ( TempNorm > 1.0d0 ) then
!!$!        write( 6, * ) 'ERROR: In ', trim( ModuleName )
!!$!        write( 6, * ) 'ERROR: In ThermoPropSublPressure'
!!$!        write( 6, * ) 'ERROR: Temperature is too high. ', TempCur
!!$!        stop
!!$!      end if
!!$!
!!$!      Tmp  =                                               &
!!$!        &   ( - 14.740846d0  * ( 1.0d0 - TempNorm )        &
!!$!        &     +  2.4327015d0 * ( 1.0d0 - TempNorm )**1.9   &
!!$!        &     -  5.3061778d0 * ( 1.0d0 - TempNorm )**2.9 )
!!$!      DTmpDT =                                                     &
!!$!        &   (           14.740846d0  *                             &
!!$!        &     - 1.9d0 *  2.4327015d0 * ( 1.0d0 - TempNorm )**0.9   &
!!$!        &     + 2.9d0 *  5.3061778d0 * ( 1.0d0 - TempNorm )**1.9 ) &
!!$!        & / TempT
!!$!      DPDT =                                               &
!!$!        &   0.51795d6 * exp( Tmp / TempNorm )              &
!!$!        & * ( DTmpDT / TempNorm - Tmp / ( TempNorm**2 ) )
!!$!      call ThermoPropSublPressure( TempCur, PressCur )
!!$!      !
!!$!      DelTemp = ( Press - PressCur ) / DPDT
!!$!      TempNext = TempCur + DelTemp
!!$!      a_Hist( iLoop ) = TempNext
!!$!      write( 6, * ) iLoop, TempCur, TempNext
!!$!      if ( abs( ( TempNext - TempCur ) / TempCur ) < ConvThreshold ) then
!!$!        exit search_subltemp
!!$!      end if
!!$!      TempCur = TempNext
!!$!    end do search_subltemp
!!$!    if ( iLoop >= NLoop ) then
!!$!      write( 6, * ) 'ERROR: In ', trim( ModuleName )
!!$!      write( 6, * ) 'ERROR: In ThermoPropSublTemperature'
!!$!      write( 6, * ) 'ERROR:   Loop does not converge'
!!$!      write( 6, * ) 'ERROR:   ', iLoop
!!$!      write( 6, * ) 'ERROR:   ', a_Hist
!!$!      stop
!!$!    end if
!!$!
!!$!    Val = TempNext
!!$
!!$
!!$
!!$    ! Bisectional method
!!$    Temp2 = TempT - 1.0d-3
!!$    call ThermoPropSublPressure( Temp2, Press2 )
!!$    if ( Press2 < Press ) then
!!$      write( 6, * ) 'ERROR: In ', trim( ModuleName )
!!$      write( 6, * ) 'ERROR: In ThermoPropSublTemperature'
!!$      write( 6, * ) 'ERROR:   Pressure seems to be too high.'
!!$      stop
!!$    end if
!!$    search_init_temperature : do
!!$      Temp1 = Temp2 - 50.0d0
!!$      call ThermoPropSublPressure( Temp1, Press1 )
!!$      if ( Press1 < Press ) then
!!$        exit
!!$      end if
!!$      Temp2 = Temp1
!!$    end do search_init_temperature
!!$    bisectional_method : do
!!$      TempCur = ( Temp1 + Temp2 ) / 2.0d0
!!$      call ThermoPropSublPressure( TempCur, PressCur )
!!$      if ( Press < PressCur ) then
!!$        Temp2 = TempCur
!!$      else
!!$        Temp1 = TempCur
!!$      end if
!!$      if ( abs( ( Temp2 - Temp1 ) / Temp1 ) < ConvThreshold ) then
!!$        exit bisectional_method
!!$      end if
!!$    end do bisectional_method
!!$    Val = ( Temp1 + Temp2 ) / 2.0d0
!!$
!!$
!!$  end subroutine ThermoPropSublTemperature

  !----------------------------------------------------------------------------
  ! vapor pressure, Eq. (4)
  !   boundary between liquid and gas
  !----------------------------------------------------------------------------

  subroutine ThermoPropSatPressure( Temp, Val )

    real(dp), intent(in ) :: Temp
    real(dp), intent(out) :: Val


    ! Local variables
    !
    integer, parameter :: NCoef = 4
    real(dp)           :: a_Coef( NCoef )
    real(dp)           :: a_Expo( NCoef )

    real(dp)           :: TempNorm

    integer            :: i

    data a_Coef / -6.12445284, 1.26327220, -0.765910082, -1.77570564 /
    data a_Expo / 1.0, 1.5, 2.5, 5.0 /


    TempNorm = Temp / TempC

    if ( TempNorm > 1.0d0 ) then
      write( 6, * ) 'ERROR: In ', trim( ModuleName )
      write( 6, * ) 'ERROR: In ThermoPropSatPressure'
      write( 6, * ) 'ERROR: Temperature is too high. ', Temp
      stop
    end if


    Val = 0.0d0
    do i = 1, NCoef
      Val = Val &
        & + a_Coef(i) * ( 1.0d0 - TempNorm )**a_Expo(i)
    end do
    Val = Val / TempNorm
    Val = PressC * exp( Val )


  end subroutine ThermoPropSatPressure

  !----------------------------------------------------------------------------
  ! ideal gas part of the reduced Helmholtz energy (Eq. (53))
  !----------------------------------------------------------------------------

  function IGAlpha( Delta, Tau ) &
    & result( HEne )

    real(dp), intent(in) :: Delta
    real(dp), intent(in) :: Tau
    real(dp)             :: HEne


    ! Local variables
    !


    HEne = log( Delta ) + a_IGA(1) * log( Tau ) &
      & + a_IGA(2) &
      & + a_IGA(3) * Tau &
      & + a_IGA(4) / Tau &
      & + a_IGA(5) / Tau**2 &
      & + a_IGA(6) / Tau**3 &
      & + a_IGA(7) * log( 1.0d0 - exp( - a_IGA(8) * Tau ) )


  end function IGAlpha

  !----------------------------------------------------------------------------
  ! delta derivative of ideal gas part of the reduced Helmholtz energy
  !----------------------------------------------------------------------------

  function IGDAlphaDDelta( Delta ) &
    & result( Val )

    real(dp), intent(in) :: Delta
    real(dp)             :: Val


    ! Local variables
    !


    Val = 1.0d0 / Delta


  end function IGDAlphaDDelta

  !----------------------------------------------------------------------------
  ! delta second derivative of ideal gas part of the reduced Helmholtz energy
  !----------------------------------------------------------------------------

  function IGD2AlphaDDelta2( Delta ) &
    & result( Val )

    real(dp), intent(in) :: Delta
    real(dp)             :: Val


    ! Local variables
    !

    Val = - 1.0d0 / Delta**2


  end function IGD2AlphaDDelta2

  !----------------------------------------------------------------------------
  ! tau derivative of ideal gas part of the reduced Helmholtz energy (Eq. (79))
  !----------------------------------------------------------------------------

  function IGDAlphaDTau( Tau ) &
    & result( Val )

    real(dp), intent(in) :: Tau
    real(dp)             :: Val


    ! Local variables
    !


    Val =         a_IGA(1) / Tau    &
      & +         a_IGA(3)          &
      & -         a_IGA(4) / Tau**2 &
      & - 2.0d0 * a_IGA(5) / Tau**3 &
      & - 3.0d0 * a_IGA(6) / Tau**4 &
      & + a_IGA(7) * a_IGA(8) / ( exp( a_IGA(8) * Tau ) - 1.0d0 )


  end function IGDAlphaDTau

  !----------------------------------------------------------------------------
  ! tau second derivative of ideal gas part of the reduced Helmholtz energy
  ! (Eq. (89))
  !----------------------------------------------------------------------------

  function IGD2AlphaDTau2( Tau ) &  
    & result( Val )

    real(dp), intent(in) :: Tau
    real(dp)             :: Val


    ! Local variables
    !
    integer :: i


    Val = -         a_IGA(1)  / Tau**2 &
      &   +  2.0d0 * a_IGA(4) / Tau**3 &
      &   +  6.0d0 * a_IGA(5) / Tau**4 &
      &   + 12.0d0 * a_IGA(6) / Tau**5 &
      &   - a_IGA(7) * a_IGA(8)**2 &
      &     * exp( a_IGA(8) * Tau ) / ( exp( a_IGA(8) * Tau ) - 1.0d0 )**2


  end function IGD2AlphaDTau2

  !----------------------------------------------------------------------------
  ! delta and tau derivative of ideal gas part of the reduced Helmholtz energy
  !----------------------------------------------------------------------------

  function IGD2AlphaDDeltaDTau( ) &
    & result( Val )

    real(dp)             :: Val


    ! Local variables
    !


    Val = 0.0d0


  end function IGD2AlphaDDeltaDTau

  !----------------------------------------------------------------------------
  ! residual part of the reduced Helmholtz energy (Eq. (55))
  !----------------------------------------------------------------------------

  function ResAlpha( Delta, Tau ) &
    & result( HEne )

    real(dp), intent(in) :: Delta
    real(dp), intent(in) :: Tau
    real(dp)             :: HEne


    ! Local variables
    !
    integer             :: k


    HEne = 0.0d0
    do k = 1, 6
      HEne = HEne &
        & + a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k)
    end do
    do k = 7, 32
      HEne = HEne &
        & + a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k) &
        &   * exp( - Delta**a_ResSmallL(k) )
    end do
    do k = 33, 36
      HEne = HEne &
        & + a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k) &
        &   * exp( - a_ResPhi (k) * ( Delta - 1.0d0         )**2       &
        &          - a_ResBeta(k) * ( Tau   - a_ResGamma(k) )**2 )
    end do


  end function ResAlpha

  !----------------------------------------------------------------------------
  ! delta derivative of residual part of the reduced Helmholtz energy
  ! (Eq. (81))
  !----------------------------------------------------------------------------

  function ResDAlphaDDelta( Delta, Tau ) &
    & result( HEne )

    real(dp), intent(in) :: Delta
    real(dp), intent(in) :: Tau
    real(dp)             :: HEne


    ! Local variables
    !
    integer             :: k


    HEne = 0.0d0
    do k = 1, 6
      HEne = HEne &
        & + a_ResSmallI(k) * a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k)
    end do
    do k = 7, 32
      HEne = HEne &
        & + a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k) &
        &   * exp( - Delta**a_ResSmallL(k) )                           &
        &   * ( a_ResSmallI(k) - a_ResSmallL(k) * Delta**a_ResSmallL(k) )
    end do
    do k = 33, 36
      HEne = HEne &
        & + a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k) &
        &   * exp( - a_ResPhi (k) * ( Delta - 1.0d0         )**2       &
        &          - a_ResBeta(k) * ( Tau   - a_ResGamma(k) )**2 )     &
        &   * ( a_ResSmallI(k) - 2.0d0 * Delta * a_ResPhi(k) * ( Delta - 1.0d0 ) )
    end do

    HEne = HEne / Delta


  end function ResDAlphaDDelta

  !----------------------------------------------------------------------------
  ! second delta derivative of residual part of the reduced Helmholtz energy
  ! (Eq. (82))
  !----------------------------------------------------------------------------

  function ResD2AlphaDDelta2( Delta, Tau ) &
    & result( HEne )

    real(dp), intent(in) :: Delta
    real(dp), intent(in) :: Tau
    real(dp)             :: HEne


    ! Local variables
    !
    integer             :: k


    HEne = 0.0d0
    do k = 1, 6
      HEne = HEne &
        & + a_ResSmallI(k) * ( a_ResSmallI(k) - 1 ) &
        &   * a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k)
    end do
    do k = 7, 32
      HEne = HEne &
        & + a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k) &
        &   * exp( - Delta**a_ResSmallL(k) )                           &
        &   * (   ( a_ResSmallI(k) - a_ResSmallL(k) * Delta**a_ResSmallL(k) ) &
        &       * ( a_ResSmallI(k) - 1 - a_ResSmallL(k) * Delta**a_ResSmallL(k) ) &
        &       - a_ResSmallL(k)**2 * Delta**a_ResSmallL(k) )
    end do
    do k = 33, 36
      HEne = HEne &
        & + a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k) &
        &   * exp( - a_ResPhi (k) * ( Delta - 1.0d0         )**2       &
        &          - a_ResBeta(k) * ( Tau   - a_ResGamma(k) )**2 )     &
        &   * (   ( a_ResSmallI(k) - 2.0d0 * a_ResPhi(k) * Delta * ( Delta - 1.0d0 ) )**2 &
        &       - a_ResSmallI(k) - 2.0d0 * Delta**2 * a_ResPhi(k) )
    end do

    HEne = HEne / Delta**2


  end function ResD2AlphaDDelta2

  !----------------------------------------------------------------------------
  ! tau derivative of residual part of the reduced Helmholtz energy (Eq. (84))
  !----------------------------------------------------------------------------

  function ResDAlphaDTau( Delta, Tau ) &
    & result( HEne )

    real(dp), intent(in) :: Delta
    real(dp), intent(in) :: Tau
    real(dp)             :: HEne


    ! Local variables
    !
    integer             :: k


    HEne = 0.0d0
    do k = 1, 6
      HEne = HEne &
        & + a_ResSmallJ(k) * a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k)
    end do
    do k = 7, 32
      HEne = HEne &
        & + a_ResSmallJ(k) * a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k) &
        &   * exp( - Delta**a_ResSmallL(k) )
    end do
    do k = 33, 36
      HEne = HEne &
        & + a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k) &
        &   * exp( - a_ResPhi (k) * ( Delta - 1.0d0         )**2       &
        &          - a_ResBeta(k) * ( Tau   - a_ResGamma(k) )**2 )     &
        &   * ( a_ResSmallJ(k) - 2.0d0 * Tau * a_ResBeta(k) * ( Tau - a_ResGamma(k) ) )
    end do

    HEne = HEne / Tau


  end function ResDAlphaDTau

  !----------------------------------------------------------------------------
  ! second tau derivative of residual part of the reduced Helmholtz energy
  ! (Eq. (85))
  !----------------------------------------------------------------------------

  function ResD2AlphaDTau2( Delta, Tau ) &
    & result( HEne )

    real(dp), intent(in) :: Delta
    real(dp), intent(in) :: Tau
    real(dp)             :: HEne


    ! Local variables
    !
    integer             :: k


    HEne = 0.0d0
    do k = 1, 6
      HEne = HEne &
        & + a_ResSmallJ(k) * ( a_ResSmallJ(k) - 1.0d0 ) &
        &   * a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k)
    end do
    do k = 7, 32
      HEne = HEne &
        & + a_ResSmallJ(k) * ( a_ResSmallJ(k) - 1.0d0 ) &
        &   * a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k) &
        &   * exp( - Delta**a_ResSmallL(k) )
    end do
    do k = 33, 36
      HEne = HEne &
        & + a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k) &
        &   * exp( - a_ResPhi (k) * ( Delta - 1.0d0         )**2       &
        &          - a_ResBeta(k) * ( Tau   - a_ResGamma(k) )**2 )     &
        &   * (   ( a_ResSmallJ(k) - 2.0d0 * Tau * a_ResBeta(k) * ( Tau   - a_ResGamma(k) ) )**2 &
        &       - a_ResSmallJ(k) - 2.0d0 * Tau**2 * a_ResBeta(k) )
    end do

    HEne = HEne / Tau**2


  end function ResD2AlphaDTau2

  !----------------------------------------------------------------------------
  ! delta and tau derivative of residual part of the reduced Helmholtz energy
  ! (Eq. (86))
  !----------------------------------------------------------------------------

  function ResD2AlphaDDeltaDTau( Delta, Tau ) &
    & result( HEne )

    real(dp), intent(in) :: Delta
    real(dp), intent(in) :: Tau
    real(dp)             :: HEne


    ! Local variables
    !
    integer             :: k


    HEne = 0.0d0
    do k = 1, 6
      HEne = HEne &
        & + a_ResSmallI(k) * a_ResSmallJ(k) * a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k)
    end do
    do k = 7, 32
      HEne = HEne &
        & + a_ResSmallJ(k) * a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k) &
        &   * exp( - Delta**a_ResSmallL(k) )                           &
        &   * ( a_ResSmallI(k) - a_ResSmallL(k) * Delta**a_ResSmallL(k) )
    end do
    do k = 33, 36
      HEne = HEne &
        & + a_ResCapN(k) * Delta**a_ResSmallI(k) * Tau**a_ResSmallJ(k) &
        &   * exp( - a_ResPhi (k) * ( Delta - 1.0d0         )**2       &
        &          - a_ResBeta(k) * ( Tau   - a_ResGamma(k) )**2 )     &
        &   * ( a_ResSmallI(k) - 2.0d0 * Delta * a_ResPhi(k) * ( Delta - 1.0d0 ) ) &
        &   * ( a_ResSmallJ(k) - 2.0d0 * Tau * a_ResBeta(k) * ( Tau - a_ResGamma(k) ) )
    end do

    HEne = HEne / ( Delta * Tau )


  end function ResD2AlphaDDeltaDTau

  !----------------------------------------------------------------------------

end module thermo_prop_n2_s2000
